"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const p_event_1 = __importDefault(require("p-event"));
const config_1 = require("../components/config");
const create_1 = require("../components/config/create");
const logger_1 = require("../components/logger");
const flush_1 = require("../components/logger/flush");
const history_1 = require("../components/logger/history");
const startup_message_1 = require("../components/logger/startup-message");
const schedule_notification_1 = require("../components/notification/schedule-notification");
const start_message_1 = require("../components/notification/start-message");
const summary_1 = require("../components/summary");
const context_1 = require("../context");
const events_1 = __importDefault(require("../events"));
const flag_1 = require("../flag");
const summary_notification_1 = require("../jobs/summary-notification");
const loaders_1 = __importDefault(require("../loaders"));
const looper_1 = require("../looper");
const symon_1 = __importDefault(require("../symon"));
const events_2 = require("../utils/events");
const pino_1 = require("../utils/pino");
const public_ip_1 = require("../utils/public-ip");
const sentry_1 = require("../plugins/sentry");
const node_1 = require("@sentry/node");
const probe_1 = require("../components/config/probe");
const em = (0, events_2.getEventEmitter)();
let symonClient;
class Monika extends core_1.Command {
    async catch(error) {
        super.catch(error);
        if ((0, context_1.getContext)().flags.sentryDSN !== undefined) {
            (0, node_1.captureException)(error);
        }
        if (symonClient) {
            await symonClient.sendStatus({ isOnline: false });
        }
        if (error instanceof core_1.Errors.ExitError) {
            return core_1.Errors.handle(error);
        }
        if (error.message.includes('EEXIT: 0')) {
            // this is normal exit, for example after running with --version,
            // not an error so just quit immediately
            // eslint-disable-next-line n/no-process-exit, unicorn/no-process-exit
            process.exit(0);
        }
        throw error;
    }
    async finally() {
        if ((0, context_1.getContext)().flags.sentryDSN !== undefined) {
            await (0, node_1.flush)(2000);
            await (0, node_1.close)();
        }
    }
    async run() {
        const cmd = await this.parse(Monika);
        const flags = (0, flag_1.sanitizeFlags)(cmd.flags);
        (0, context_1.setContext)({ flags, userAgent: this.config.userAgent });
        try {
            if (flags.version) {
                this.log(this.config.userAgent);
                this.exit(0);
            }
            if (flags['create-config']) {
                await (0, create_1.createConfig)();
                return;
            }
            if (flags.logs) {
                await (0, logger_1.printAllLogs)();
                return;
            }
            if (flags.flush) {
                await (0, flush_1.flush)();
                return;
            }
            if (flags.summary) {
                await (0, summary_1.printSummary)();
                return;
            }
            const isSymonMode = (0, config_1.isSymonModeFrom)(flags);
            if (!isSymonMode) {
                await (0, config_1.initConfig)();
            }
            await (0, loaders_1.default)(flags, this.config);
            if (flags['skip-start-message'] === false) {
                await logRunningInfo({ isSymonMode, isVerbose: flags.verbose });
            }
            if (isSymonMode) {
                symonClient = new symon_1.default(flags);
                await symonClient.initiate();
            }
            if (flags.sentryDSN !== undefined) {
                pino_1.log.info('Sentry is enabled for error reporting');
                (0, sentry_1.initSentry)({
                    dsn: flags.sentryDSN,
                    monikaVersion: this.config.version,
                });
            }
            let isFirstRun = true;
            for (;;) {
                const config = (0, config_1.getValidatedConfig)();
                const probes = (0, probe_1.getProbes)();
                // emit the sanitized probe
                em.emit(events_1.default.config.sanitized, probes);
                // save some data into files for later
                (0, summary_notification_1.savePidFile)(flags.config, config);
                deprecationHandler(config);
                (0, startup_message_1.logStartupMessage)({
                    config,
                    flags,
                    isFirstRun,
                });
                const controller = new AbortController();
                const { signal } = controller;
                const { notifications } = config;
                (0, looper_1.startProbing)({
                    notifications,
                    probes,
                    signal,
                });
                if ((0, context_1.getContext)().isTest) {
                    break;
                }
                if (flags['skip-start-message'] === false) {
                    // if skipping, skip also startup notification
                    (0, start_message_1.sendMonikaStartMessage)(notifications).catch((error) => pino_1.log.error(error.message));
                }
                // schedule status update notification
                (0, schedule_notification_1.scheduleSummaryNotification)({ config, flags });
                isFirstRun = false;
                // block the loop until receives config updated event
                // eslint-disable-next-line no-await-in-loop
                await (0, p_event_1.default)(em, events_1.default.config.updated);
                controller.abort('Monika configuration updated');
            }
        }
        catch (error) {
            this.error(error?.message, { exit: 1 });
        }
        finally {
            (0, history_1.closeLog)();
        }
    }
}
Monika.description = 'Monika command line monitoring tool';
Monika.examples = [
    'monika',
    'monika --logs',
    'monika -r 1 --id "weather, stocks, 5, 7"',
    'monika --create-config',
    'monika --config https://raw.githubusercontent.com/hyperjumptech/monika/main/monika.example.yml --config-interval 900',
];
Monika.flags = flag_1.flags;
Monika.id = 'monika';
exports.default = Monika;
async function logRunningInfo({ isVerbose, isSymonMode }) {
    if (!isVerbose && !isSymonMode) {
        pino_1.log.info('Monika is running.');
        return;
    }
    try {
        const { city, hostname, isp, privateIp, publicIp } = await (0, public_ip_1.fetchAndCacheNetworkInfo)();
        pino_1.log.info(`Monika is running from: ${city} - ${isp} (${publicIp}) - ${hostname} (${privateIp})`);
    }
    catch (error) {
        pino_1.log.warn(`Failed to obtain location/ISP info. Got: ${error}`);
    }
}
function deprecationHandler(config) {
    const showDeprecateMsg = {
        query: false,
    };
    const checkedConfig = {
        ...config,
        probes: config.probes?.map((probe) => ({
            ...probe,
            requests: probe.requests?.map((request) => ({
                ...request,
                alert: request.alerts?.map((alert) => {
                    if (alert.query) {
                        showDeprecateMsg.query = true;
                        return { ...alert, assertion: alert.query };
                    }
                    return alert;
                }),
            })),
            alerts: probe.alerts?.map((alert) => {
                if (alert.query) {
                    showDeprecateMsg.query = true;
                    return { ...alert, assertion: alert.query };
                }
                return alert;
            }),
        })),
    };
    if (showDeprecateMsg.query) {
        pino_1.log.warn('"alerts.query" is deprecated. Please use "alerts.assertion"');
    }
    return checkedConfig;
}
/**
 * Show Exit Message
 */
process.on('SIGINT', async () => {
    pino_1.log.info('Thank you for using Monika!');
    pino_1.log.info('We need your help to make Monika better.');
    pino_1.log.info('Can you give us some feedback by clicking this link https://github.com/hyperjumptech/monika/discussions?\n');
    if (symonClient) {
        await symonClient.sendStatus({ isOnline: false });
        await symonClient.stop();
    }
    await (0, node_1.flush)(2000);
    em.emit(events_1.default.application.terminated);
    // eslint-disable-next-line n/no-process-exit
    process.exit(process.exitCode);
});
//# sourceMappingURL=monika.js.map