"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_child_process_1 = __importDefault(require("node:child_process"));
const node_fs_1 = require("node:fs");
const promises_1 = require("node:fs/promises");
const node_os_1 = __importDefault(require("node:os"));
const test_1 = require("@oclif/test");
const sinon_1 = require("sinon");
const context_1 = require("../../context");
const catch_error_handler_1 = require("../../utils/catch-error-handler");
const create_1 = require("./create");
describe('Create config', () => {
    describe('Open Monika config generator', () => {
        let osTypeStub;
        let spawnSyncStub;
        beforeEach(() => {
            osTypeStub = (0, sinon_1.stub)(node_os_1.default, 'type');
            spawnSyncStub = (0, sinon_1.stub)(node_child_process_1.default, 'spawnSync');
        });
        afterEach(() => {
            osTypeStub.restore();
            spawnSyncStub.restore();
        });
        it('should open Monika config generator using MacOS', async () => {
            // arrange
            osTypeStub.returns('Darwin');
            // act
            await (0, create_1.createConfig)();
            // assert
            const command = spawnSyncStub.args[0][0];
            const url = spawnSyncStub.args[0][1][0];
            (0, test_1.expect)(command).eq('open');
            (0, test_1.expect)(url).eq('https://hyperjumptech.github.io/monika-config-generator/');
        });
        it('should open Monika config generator using Linux', async () => {
            // arrange
            osTypeStub.returns('Linux');
            // act
            await (0, create_1.createConfig)();
            // assert
            const command = spawnSyncStub.args[0][0];
            const url = spawnSyncStub.args[0][1][0];
            (0, test_1.expect)(command).eq('xdg-open');
            (0, test_1.expect)(url).eq('https://hyperjumptech.github.io/monika-config-generator/');
        });
        it('should open Monika config generator using Windows', async () => {
            // arrange
            osTypeStub.returns('Windows NT');
            // act
            await (0, create_1.createConfig)();
            // assert
            const command = spawnSyncStub.args[0][0];
            const url = spawnSyncStub.args[0][1][0];
            (0, test_1.expect)(command).eq('start');
            (0, test_1.expect)(url).eq('https://hyperjumptech.github.io/monika-config-generator/');
        });
        it('should throw an error on unhandled OS', async () => {
            // arrange
            let errorMessage = '';
            osTypeStub.returns('FreeBSD');
            // act
            try {
                await (0, create_1.createConfig)();
            }
            catch (error) {
                errorMessage = (0, catch_error_handler_1.getErrorMessage)(error);
            }
            // assert
            (0, test_1.expect)(errorMessage).to.include('Unknown operating system');
        });
    });
    describe('Converter', () => {
        afterEach(() => {
            (0, context_1.resetContext)();
        });
        it('should throw an error if file path is not found', async () => {
            // arrange
            const filepath = './example.har';
            (0, context_1.setContext)({ flags: { ...(0, context_1.getContext)().flags, har: filepath } });
            let errorMessage = '';
            // act
            try {
                await (0, create_1.createConfig)();
            }
            catch (error) {
                errorMessage = (0, catch_error_handler_1.getErrorMessage)(error);
            }
            // assert
            (0, test_1.expect)(errorMessage).to.include("Couldn't found");
        });
        it('should convert from har', async () => {
            // arrange
            const output = 'monika.har.yml';
            (0, context_1.setContext)({
                flags: {
                    ...(0, context_1.getContext)().flags,
                    force: true,
                    output,
                    har: './src/components/config/__tests__/form_encoded.har',
                },
            });
            // act
            await (0, create_1.createConfig)();
            // assert
            const generated = await (0, promises_1.readFile)(output, { encoding: 'utf8' });
            const expected = await (0, promises_1.readFile)('./src/components/config/__tests__/expected.har.yml', { encoding: 'utf8' });
            (0, test_1.expect)(generated).eq(expected);
            await (0, promises_1.rm)(output);
        });
        it('should convert from insomnia', async () => {
            // arrange
            const output = 'monika.insomnia.yml';
            (0, context_1.setContext)({
                flags: {
                    ...(0, context_1.getContext)().flags,
                    force: true,
                    insomnia: './src/components/config/__tests__/petstore.insomnia.yaml',
                    output,
                },
            });
            // act
            await (0, create_1.createConfig)();
            // assert
            const generated = await (0, promises_1.readFile)(output, {
                encoding: 'utf8',
            });
            const expected = await (0, promises_1.readFile)('./src/components/config/__tests__/expected.insomnia.yml', { encoding: 'utf8' });
            (0, test_1.expect)(generated).eq(expected);
            await (0, promises_1.rm)(output);
        });
        it('should convert from basic postman file v2.0', async () => {
            // arrange
            const output = 'monika.postman-basic.yml';
            (0, context_1.setContext)({
                flags: {
                    ...(0, context_1.getContext)().flags,
                    force: true,
                    postman: './src/components/config/__tests__/mock_files/basic-postman_collection-v2.0.json',
                    output,
                },
            });
            // act
            await (0, create_1.createConfig)();
            // assert
            const { generated, expected } = await getPostmanConfig();
            (0, test_1.expect)(generated).eq(expected);
            await (0, promises_1.rm)(output);
        });
        it('should convert from grouped postman file v2.0', async () => {
            // arrange
            const output = 'monika.postman-grouped.yml';
            (0, context_1.setContext)({
                flags: {
                    ...(0, context_1.getContext)().flags,
                    force: true,
                    postman: './src/components/config/__tests__/mock_files/grouped-postman_collection-v2.0.json',
                    output,
                },
            });
            // act
            await (0, create_1.createConfig)();
            // assert
            const { generated, expected } = await getPostmanConfig(true);
            (0, test_1.expect)(generated).eq(expected);
            await (0, promises_1.rm)(output);
        });
        it('should convert from basic postman file v2.1', async () => {
            // arrange
            const output = 'monika.postman-basic.yml';
            (0, context_1.setContext)({
                flags: {
                    ...(0, context_1.getContext)().flags,
                    force: true,
                    postman: './src/components/config/__tests__/mock_files/basic-postman_collection-v2.0.json',
                    output: 'monika.postman-basic.yml',
                },
            });
            // act
            await (0, create_1.createConfig)();
            // assert
            const { generated, expected } = await getPostmanConfig();
            (0, test_1.expect)(generated).eq(expected);
            await (0, promises_1.rm)(output);
        });
        it('should convert from grouped postman file v2.1', async () => {
            // arrange
            const output = 'monika.postman-grouped.yml';
            (0, context_1.setContext)({
                flags: {
                    ...(0, context_1.getContext)().flags,
                    force: true,
                    postman: './src/components/config/__tests__/mock_files/grouped-postman_collection-v2.1.json',
                    output,
                },
            });
            // act
            await (0, create_1.createConfig)();
            // assert
            const { generated, expected } = await getPostmanConfig(true);
            (0, test_1.expect)(generated).eq(expected);
            await (0, promises_1.rm)(output);
        });
        it('should convert from sitemap', async () => {
            // arrange
            const output = 'monika.sitemap.yml';
            (0, context_1.setContext)({
                flags: {
                    ...(0, context_1.getContext)().flags,
                    force: true,
                    sitemap: './src/components/config/__tests__/sitemap.xml',
                    output,
                },
            });
            // act
            await (0, create_1.createConfig)();
            // assert
            const generated = await (0, promises_1.readFile)('monika.sitemap.yml', {
                encoding: 'utf8',
            });
            const expected = await (0, promises_1.readFile)('./src/components/config/__tests__/expected.sitemap.yml', { encoding: 'utf8' });
            (0, test_1.expect)(generated).eq(expected);
            await (0, promises_1.rm)(output);
        });
        it('should convert from sitemap - one probe', async () => {
            // arrange
            const output = 'monika.sitemap.yml';
            (0, context_1.setContext)({
                flags: {
                    ...(0, context_1.getContext)().flags,
                    force: true,
                    sitemap: './src/components/config/__tests__/sitemap.xml',
                    output,
                    'one-probe': true,
                },
            });
            // act
            await (0, create_1.createConfig)();
            // assert
            const generated = await (0, promises_1.readFile)('monika.sitemap.yml', {
                encoding: 'utf8',
            });
            const expected = await (0, promises_1.readFile)('./src/components/config/__tests__/expected.sitemap-oneprobe.yml', { encoding: 'utf8' });
            (0, test_1.expect)(generated).eq(expected);
            await (0, promises_1.rm)(output);
        });
        it('should convert from text', async () => {
            // arrange
            const output = 'monika.textfile.yml';
            (0, context_1.setContext)({
                flags: {
                    ...(0, context_1.getContext)().flags,
                    force: true,
                    text: './src/components/config/__tests__/textfile',
                    output,
                },
            });
            // act
            await (0, create_1.createConfig)();
            // assert
            const generated = await (0, promises_1.readFile)('monika.textfile.yml', {
                encoding: 'utf8',
            });
            const expected = await (0, promises_1.readFile)('./src/components/config/__tests__/expected.textfile.yml', { encoding: 'utf8' });
            (0, test_1.expect)(generated).eq(expected);
            await (0, promises_1.rm)(output);
        });
    });
    describe('Writer', () => {
        beforeEach(async () => {
            if ((0, node_fs_1.existsSync)('monika.yml')) {
                await (0, promises_1.rename)('monika.yml', 'monika.backup.yml');
            }
        });
        afterEach(async () => {
            if ((0, node_fs_1.existsSync)('monika.backup.yml')) {
                await (0, promises_1.rename)('monika.backup.yml', 'monika.yml');
            }
            (0, context_1.resetContext)();
        });
        it('should not prompt to overwrite existing file', async () => {
            // arrange
            (0, context_1.setContext)({
                flags: {
                    ...(0, context_1.getContext)().flags,
                    force: true,
                    text: './src/components/config/__tests__/textfile',
                },
            });
            await (0, promises_1.writeFile)('monika.yml', '', { encoding: 'utf8' });
            // act
            await (0, create_1.createConfig)();
            // assert
            (0, test_1.expect)((0, node_fs_1.existsSync)('monika.yml')).eq(true);
            await (0, promises_1.rm)('monika.yml');
        });
        it('should write config to the output flag', async () => {
            // arrange
            if ((0, node_fs_1.existsSync)('monika.yaml')) {
                await (0, promises_1.rename)('monika.yaml', 'monika.backup.yaml');
            }
            (0, context_1.setContext)({
                flags: {
                    ...(0, context_1.getContext)().flags,
                    force: true,
                    output: 'monika.yaml',
                    text: './src/components/config/__tests__/textfile',
                },
            });
            await (0, promises_1.writeFile)('monika.yaml', '', { encoding: 'utf8' });
            // act
            await (0, create_1.createConfig)();
            // assert
            (0, test_1.expect)((0, node_fs_1.existsSync)('monika.yaml')).eq(true);
            await (0, promises_1.rm)('monika.yaml');
            if ((0, node_fs_1.existsSync)('monika.backup.yaml')) {
                await (0, promises_1.rename)('monika.backup.yaml', 'monika.yaml');
            }
        });
    });
});
async function getPostmanConfig(isGrouped = false) {
    if (isGrouped) {
        const generated = await (0, promises_1.readFile)('monika.postman-grouped.yml', {
            encoding: 'utf8',
        });
        const expected = await (0, promises_1.readFile)('./src/components/config/__tests__/mock_files/expected.postman-grouped.yml', { encoding: 'utf8' });
        return { generated, expected };
    }
    const generated = await (0, promises_1.readFile)('monika.postman-basic.yml', {
        encoding: 'utf8',
    });
    const expected = await (0, promises_1.readFile)('./src/components/config/__tests__/mock_files/expected.postman-basic.yml', { encoding: 'utf8' });
    return { generated, expected };
}
//# sourceMappingURL=create.test.js.map