"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isSymonModeFrom = exports.updateConfig = exports.getValidatedConfig = exports.initConfig = void 0;
const promises_1 = require("node:fs/promises");
const events_1 = __importDefault(require("../../events"));
const context_1 = require("../../context");
const events_2 = require("../../utils/events");
const http_1 = require("../..//utils/http");
const pino_1 = require("../../utils/pino");
const get_1 = require("./get");
const probe_1 = require("./probe");
const sanitize_1 = require("./sanitize");
const validate_1 = require("./validate");
const compact_config_1 = require("./compact-config");
async function initConfig() {
    const { flags } = (0, context_1.getContext)();
    const hasConfig = flags.config.length > 0 ||
        flags.har ||
        flags.insomnia ||
        flags.postman ||
        flags.sitemap ||
        flags.text;
    if (!hasConfig) {
        await createExampleConfigFile();
    }
    const config = await (0, get_1.getRawConfig)();
    await updateConfig(config);
}
exports.initConfig = initConfig;
async function createExampleConfigFile() {
    pino_1.log.info('No Monika configuration available, initializing...');
    const outputFilePath = (0, context_1.getContext)().flags['config-filename'];
    const url = 'https://raw.githubusercontent.com/hyperjumptech/monika/main/monika.example.yml';
    try {
        const resp = await (0, http_1.sendHttpRequest)({ url });
        await (0, promises_1.writeFile)(outputFilePath, resp.data, { encoding: 'utf8' });
    }
    catch {
        const ymlConfig = `
    probes:
    - id: '1'
      requests:
        - url: http://example.com
    
    db_limit:
      max_db_size: 1000000000
      deleted_data: 1
      cron_schedule: '*/1 * * * *'
    `;
        await (0, promises_1.writeFile)(outputFilePath, ymlConfig, { encoding: 'utf8' });
    }
    (0, context_1.setContext)({ flags: { ...(0, context_1.getContext)().flags, config: [outputFilePath] } });
    pino_1.log.info(`${outputFilePath} file has been created in this directory. You can change the URL to probe and other configurations in that ${outputFilePath} file.`);
}
function getValidatedConfig() {
    const { config, flags } = (0, context_1.getContext)();
    if (!config) {
        if (!isSymonModeFrom(flags)) {
            throw new Error('Configuration setup has not been run yet');
        }
        return (0, sanitize_1.sanitizeConfig)({
            probes: [],
        });
    }
    return { ...config, probes: (0, probe_1.getProbes)() };
}
exports.getValidatedConfig = getValidatedConfig;
async function updateConfig(config) {
    const validatedConfig = await (0, validate_1.validateConfig)(config);
    const sanitizedConfig = (0, sanitize_1.sanitizeConfig)(validatedConfig);
    const hasConfigChange = (0, context_1.getContext)().config?.version !== sanitizedConfig.version;
    if (!hasConfigChange) {
        return;
    }
    const isInitalSetup = (0, context_1.getContext)().config?.version === undefined;
    if (!isInitalSetup) {
        pino_1.log.info('Config changes. Updating config...');
    }
    (0, context_1.setContext)({
        config: sanitizedConfig,
    });
    let { probes } = sanitizedConfig;
    if ((0, context_1.getContext)().flags['compact-probes']) {
        probes = (0, compact_config_1.compactProbes)(probes);
    }
    (0, probe_1.setProbes)(probes);
    (0, events_2.getEventEmitter)().emit(events_1.default.config.updated);
}
exports.updateConfig = updateConfig;
function isSymonModeFrom({ symonKey, symonUrl, }) {
    return Boolean(symonUrl) && Boolean(symonKey);
}
exports.isSymonModeFrom = isSymonModeFrom;
//# sourceMappingURL=index.js.map