"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const promises_1 = require("node:fs/promises");
const node_fs_1 = require("node:fs");
const test_1 = require("@oclif/test");
const chai_1 = __importDefault(require("chai"));
const chai_spies_1 = __importDefault(require("chai-spies"));
const msw_1 = require("msw");
const node_1 = require("msw/node");
const context_1 = require("../../context");
const events_1 = __importDefault(require("../../events"));
const flag_1 = require("../../flag");
const catch_error_handler_1 = require("../../utils/catch-error-handler");
const events_2 = require("../../utils/events");
const pino_1 = require("../../utils/pino");
const sanitize_1 = require("./sanitize");
const _1 = require(".");
const validation_1 = require("./validation");
describe('getValidatedConfig', () => {
    beforeEach(() => {
        (0, context_1.resetContext)();
    });
    it('should throw error when config is empty', () => {
        (0, test_1.expect)(() => (0, _1.getValidatedConfig)()).to.throw('Configuration setup has not been run yet');
    });
    it('should not throw error when config is empty in symon mode', () => {
        // arrange
        (0, context_1.setContext)({
            flags: (0, flag_1.sanitizeFlags)({
                symonKey: 'bDF8j',
                symonUrl: 'https://example.com',
            }),
        });
        // act & assert
        (0, test_1.expect)((0, _1.getValidatedConfig)().version).eq('efde57d3b97ff787384fc2afde824615');
    });
    it('should return empty array for probes when config is empty in Symon mode', () => {
        // arrange
        (0, context_1.setContext)({
            flags: (0, flag_1.sanitizeFlags)({
                symonKey: 'bDF8j',
                symonUrl: 'https://example.com',
            }),
        });
        // act and assert
        (0, test_1.expect)((0, _1.getValidatedConfig)().probes).deep.eq([]);
    });
});
describe('isSymonModeFrom', () => {
    it('should return true', () => {
        // act
        const isSymonMode = (0, _1.isSymonModeFrom)({
            symonKey: 'secret-key',
            symonUrl: 'https://example.com',
        });
        // assert
        (0, test_1.expect)(isSymonMode).eq(true);
    });
    it('should return false when symon key and url is empty', () => {
        // act
        const isSymonMode = (0, _1.isSymonModeFrom)({
            symonKey: '',
            symonUrl: '',
        });
        // assert
        (0, test_1.expect)(isSymonMode).eq(false);
    });
    it('should return false when symon key is empty', () => {
        // act
        const isSymonMode = (0, _1.isSymonModeFrom)({
            symonKey: '',
            symonUrl: 'https://example.com',
        });
        // assert
        (0, test_1.expect)(isSymonMode).eq(false);
    });
    it('should return false when symon url is empty', () => {
        // act
        const isSymonMode = (0, _1.isSymonModeFrom)({
            symonKey: 'secret-key',
            symonUrl: '',
        });
        // assert
        (0, test_1.expect)(isSymonMode).eq(false);
    });
});
describe('updateConfig', () => {
    beforeEach(() => {
        (0, context_1.resetContext)();
    });
    it('should throw error when config is invalid', async () => {
        // arrange
        let errorMessage = '';
        try {
            // act
            await (0, _1.updateConfig)({ probes: [{}] });
        }
        catch (error) {
            errorMessage = (0, catch_error_handler_1.getErrorMessage)(error);
        }
        // assert
        (0, test_1.expect)(errorMessage).contain('Monika configuration is invalid');
    });
    it('should update config', async () => {
        // arrange
        const config = {
            notifications: [],
            probes: await (0, validation_1.validateProbes)([
                {
                    id: '1',
                    name: '',
                    interval: 1000,
                    requests: [
                        {
                            url: 'https://example.com',
                            body: '',
                            followRedirects: 21,
                            timeout: 1000,
                        },
                    ],
                    alerts: [],
                },
            ]),
        };
        let configFromEmitter = '';
        const eventEmitter = (0, events_2.getEventEmitter)();
        eventEmitter.once(events_1.default.config.updated, (data) => {
            configFromEmitter = data;
        });
        // act
        await (0, _1.updateConfig)(config);
        // assert
        (0, test_1.expect)((0, context_1.getContext)().config).to.deep.eq((0, sanitize_1.sanitizeConfig)(config));
        (0, test_1.expect)(configFromEmitter).not.eq('');
    });
    it('should should not update config if there is no changes', async () => {
        // arrange
        const config = {
            probes: [
                {
                    id: '1',
                    name: '',
                    interval: 1000,
                    requests: [
                        {
                            url: 'https://example.com',
                            body: '',
                            followRedirects: 21,
                            timeout: 1000,
                        },
                    ],
                    alerts: [],
                },
            ],
        };
        let configFromEmitter = '';
        const eventEmitter = (0, events_2.getEventEmitter)();
        const eventListener = (data) => {
            configFromEmitter = data;
        };
        // act
        await (0, _1.updateConfig)(config);
        eventEmitter.once(events_1.default.config.updated, eventListener);
        await (0, _1.updateConfig)(config);
        // assert
        (0, test_1.expect)(configFromEmitter).eq('');
        eventEmitter.removeListener(events_1.default.config.updated, eventListener);
    });
});
describe('Setup Config', () => {
    const server = (0, node_1.setupServer)();
    chai_1.default.use(chai_spies_1.default);
    let logMessages = [];
    before(() => {
        server.listen({ onUnhandledRequest: 'bypass' });
    });
    beforeEach(async () => {
        if ((0, node_fs_1.existsSync)('monika.yml')) {
            await (0, promises_1.rename)('monika.yml', 'monika_backup.yml');
        }
        (0, context_1.setContext)({
            ...(0, context_1.getContext)(),
            flags: { ...(0, context_1.getContext)().flags, config: [] },
        });
        chai_1.default.spy.on(pino_1.log, 'info', (message) => {
            logMessages.push(message);
        });
    });
    afterEach(async () => {
        server.resetHandlers();
        if ((0, node_fs_1.existsSync)('monika_backup.yml')) {
            await (0, promises_1.rename)('monika_backup.yml', 'monika.yml');
        }
        chai_1.default.spy.restore();
        logMessages = [];
    });
    after(() => {
        server.close();
    });
    it('should create example config if config does not exist', async () => {
        // act
        await (0, _1.initConfig)();
        // assert
        (0, test_1.expect)((0, node_fs_1.existsSync)('monika.yml')).eq(true);
    });
    it('should create local example config if the url is unreachable', async () => {
        // arrange
        server.use(msw_1.http.get('https://raw.githubusercontent.com/hyperjumptech/monika/main/monika.example.yml', () => msw_1.HttpResponse.error()));
        // act
        await (0, _1.initConfig)();
        // assert
        (0, test_1.expect)((0, node_fs_1.existsSync)('monika.yml')).eq(true);
        (0, test_1.expect)(await (0, promises_1.readFile)('monika.yml', { encoding: 'utf8' })).include('http://example.com');
    });
    it('log config changes info', async () => {
        // arrange
        server.use(msw_1.http.get('https://raw.githubusercontent.com/hyperjumptech/monika/main/monika.example.yml', () => msw_1.HttpResponse.error()));
        (0, context_1.setContext)({ config: (0, sanitize_1.sanitizeConfig)({ probes: [], version: '0.0.1' }) });
        // act
        await (0, _1.initConfig)();
        // assert
        (0, test_1.expect)(logMessages.find((message) => message === 'Config changes. Updating config...')).not.undefined;
    });
});
describe('sanitizeConfig', () => {
    beforeEach(() => {
        (0, context_1.resetContext)();
    });
    it('should sanitize probes in normal mode', () => {
        // arrange
        const config = {
            probes: [
                {
                    id: '1',
                    name: 'Test probe',
                    interval: 1000,
                    requests: [
                        {
                            url: 'https://example.com',
                            body: '',
                            followRedirects: 21,
                            timeout: 1000,
                        },
                    ],
                    alerts: [],
                },
            ],
        };
        // act
        const result = (0, sanitize_1.sanitizeConfig)(config);
        // assert
        (0, test_1.expect)(result.probes[0].alerts).to.have.lengthOf(1);
        (0, test_1.expect)(result.probes[0].alerts[0]).to.include({
            assertion: '',
            message: 'Probe not accessible',
        });
    });
    it('should sanitize probes in Symon mode', () => {
        // arrange
        (0, context_1.setContext)({
            flags: (0, flag_1.sanitizeFlags)({
                symonKey: 'test-key',
                symonUrl: 'https://example.com',
            }),
        });
        const config = {
            probes: [
                {
                    id: '1',
                    name: 'Test probe',
                    interval: 30000,
                    requests: [
                        {
                            url: 'https://example.com',
                            body: '',
                            followRedirects: 21,
                            timeout: 1000,
                        },
                    ],
                    alerts: [],
                },
            ],
        };
        // act
        const result = (0, sanitize_1.sanitizeConfig)(config);
        // assert
        (0, test_1.expect)(result.probes[0].alerts).to.have.lengthOf(0);
        (0, test_1.expect)(result.probes[0].id).to.equal('1');
        (0, test_1.expect)(result.probes[0].interval).to.equal(30000);
    });
    it('should preserve existing config properties', () => {
        // arrange
        const config = {
            version: '1.0.0',
            notifications: [{ id: 'test-1', type: 'smtp' }],
            certificate: {
                domains: ['example.com'],
                reminder: 15,
            },
            probes: [],
        };
        // act
        const result = (0, sanitize_1.sanitizeConfig)(config);
        // assert
        (0, test_1.expect)(result.version).to.equal('1.0.0');
        (0, test_1.expect)(result.notifications).to.deep.equal([{ id: 'test-1', type: 'smtp' }]);
        (0, test_1.expect)(result.certificate?.reminder).to.equal(15);
    });
});
//# sourceMappingURL=index.test.js.map