"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseHarFile = void 0;
const joi_1 = __importDefault(require("joi"));
const default_values_1 = require("../validation/validator/default-values");
const keyValValidator = joi_1.default.array().items(joi_1.default.object({ key: joi_1.default.string(), value: joi_1.default.string() }));
const convertNameValueArraysToObject = (keyVal) => {
    const obj = {};
    for (const item of keyVal) {
        if (item.name.charAt(0) !== ':') {
            obj[item.name] = item.value;
        }
    }
    return obj;
};
const postDataValidator = joi_1.default.object({
    mimeType: joi_1.default.string(),
    text: joi_1.default.string(),
    params: keyValValidator.optional(),
});
const parsePostData = (postData) => {
    if (!postData) {
        return {};
    }
    if (postData.mimeType === 'application/json') {
        return JSON.parse(postData.text);
    }
    return postData.text;
};
const parseHarFile = (fileContents) => {
    // Read file from filepath
    try {
        const entryValidator = joi_1.default.object({
            entry: joi_1.default.object({
                request: joi_1.default.object({
                    method: joi_1.default.string(),
                    url: joi_1.default.string(),
                    headers: joi_1.default.array().items(keyValValidator),
                    queryString: joi_1.default.array().items(keyValValidator),
                    postData: postDataValidator,
                }),
            }),
        });
        const { value: harJson, error } = joi_1.default.object({
            log: joi_1.default.object({
                entries: joi_1.default.array().items(entryValidator.required()),
            }),
        }).validate(JSON.parse(fileContents), { allowUnknown: true });
        if (error)
            throw new Error(`No HTTP requests in your HAR file. ${error.message}`);
        const harRequest = harJson.log.entries.map((entry) => ({
            method: entry.request.method,
            url: entry.request.url,
            headers: convertNameValueArraysToObject(entry.request.headers),
            params: convertNameValueArraysToObject(entry.request.queryString),
            body: parsePostData(entry.request.postData),
        }));
        const harConfig = {
            probes: [
                {
                    id: harRequest[0].url,
                    name: harRequest[0].url,
                    interval: default_values_1.DEFAULT_INTERVAL,
                    requests: harRequest,
                    alerts: [],
                },
            ],
        };
        return harConfig;
    }
    catch (error) {
        const parsingError = error instanceof Error ? error : new Error(`Parsing failed: ${error}.`);
        if (parsingError.name === 'SyntaxError') {
            throw new Error('Har file is in invalid JSON format!');
        }
        throw new Error(parsingError.message);
    }
};
exports.parseHarFile = parseHarFile;
//# sourceMappingURL=har.js.map