"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseInsomnia = void 0;
const js_yaml_1 = __importDefault(require("js-yaml"));
const handlebars_1 = require("handlebars");
const joi_1 = __importDefault(require("joi"));
const envValidator = joi_1.default.object({
    scheme: joi_1.default.array().items(joi_1.default.string()),
    host: joi_1.default.string(),
    // eslint-disable-next-line camelcase
    base_path: joi_1.default.string(),
});
const resourceValidator = joi_1.default.object({
    _id: joi_1.default.string(),
    authentication: joi_1.default.object({
        type: joi_1.default.string(),
        disabled: joi_1.default.boolean(),
        prefix: joi_1.default.string(),
    }),
    name: joi_1.default.string(),
    data: envValidator,
    description: joi_1.default.string().allow(''),
    url: joi_1.default.string(),
    headers: joi_1.default.array().items(joi_1.default.object({
        name: joi_1.default.string(),
        value: joi_1.default.string(),
        disabled: joi_1.default.boolean(),
    })),
    method: joi_1.default.string(),
    body: joi_1.default.object({ mimeType: joi_1.default.string(), text: joi_1.default.string() }),
    _type: joi_1.default.string(),
});
const insomniaValidator = joi_1.default.object({
    // eslint-disable-next-line camelcase
    __export_format: joi_1.default.number(),
    // eslint-disable-next-line camelcase
    __export_source: joi_1.default.string(),
    resources: joi_1.default.array().items(resourceValidator),
});
let baseUrl = '';
let environmentVariables;
function parseInsomnia(configString, format) {
    const parseResult = format === 'yaml' || format === 'yml'
        ? js_yaml_1.default.load(configString, { json: true })
        : JSON.parse(configString);
    const { error, value: insomniaData } = insomniaValidator.validate(parseResult, { allowUnknown: true });
    if (error)
        throw new Error(error?.message);
    const data = insomniaData.resources;
    if (data?.__export_format !== 4 &&
        data?.__export_source?.includes('insomnia')) {
        throw new Error('Failed to parse Insomnia collection, please use export format v4.');
    }
    const env = data
        .reverse()
        .find((res) => res._type === 'environment' &&
        res.data?.scheme &&
        res.data?.host &&
        res.data?.base_path)?.data;
    environmentVariables = env;
    baseUrl = env
        ? `${env?.scheme?.[0] ?? 'http'}://${env?.host}${env?.base_path}`
        : '';
    return mapInsomniaToConfig(data);
}
exports.parseInsomnia = parseInsomnia;
function mapInsomniaToConfig(data) {
    const res = joi_1.default.array()
        .items(resourceValidator)
        .validate(data, { allowUnknown: true }).value;
    const insomniaRequests = res.filter(({ _type, url, body, }) => _type === 'request' &&
        url &&
        // skip binary upload requests
        body?.mimeType !== 'application/octet-stream');
    const probes = insomniaRequests.map((probe) => mapInsomniaRequestToConfig(probe));
    return { probes };
}
function mapInsomniaRequestToConfig(req) {
    const { value: res } = resourceValidator.validate(req, { allowUnknown: true });
    // eslint-disable-next-line camelcase
    const url = (0, handlebars_1.compile)(res.url)({ base_url: baseUrl });
    const authorization = getAuthorizationHeader(res);
    let headers;
    if (authorization)
        headers = {
            authorization,
        };
    if (res.headers) {
        if (headers === undefined)
            headers = {};
        for (const h of res.headers) {
            headers[h.name] = h.value;
        }
    }
    return {
        id: res._id,
        name: res.name,
        description: res.description,
        requests: [
            {
                url,
                method: (res?.method ?? 'GET'),
                body: JSON.parse(res.body?.text ?? '{}'),
                timeout: 10000,
                headers,
            },
        ],
        interval: 30,
        alerts: [],
    };
}
function getAuthorizationHeader(data) {
    const { value: res } = resourceValidator.validate(data, {
        allowUnknown: true,
    });
    let authorization;
    if (res.authentication?.type === 'bearer' &&
        (res.authentication?.disabled ?? false) === false) {
        let authTemplate = res.authentication?.token;
        authTemplate = authTemplate?.replace('_.', '');
        authorization = `${res.authentication?.prefix ?? 'bearer'} ${(0, handlebars_1.compile)(authTemplate)(environmentVariables ?? {})}`;
    }
    return authorization;
}
//# sourceMappingURL=insomnia.js.map