"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateProbes = void 0;
const joi_1 = __importDefault(require("joi"));
const uuid_1 = require("uuid");
const __1 = require("../..");
const context_1 = require("../../../../context");
const looper_1 = require("../../../../looper");
const expression_parser_1 = require("../../../../utils/expression-parser");
const is_valid_url_1 = require("../../../../utils/is-valid-url");
const pino_1 = require("../../../../utils/pino");
const default_values_1 = require("./default-values");
const catch_error_handler_1 = require("../../../../utils/catch-error-handler");
const ALERT_QUERY = 'status-not-2xx';
const RESPONSE_TIME_PREFIX = 'response-time-greater-than-';
async function validateProbes(probes) {
    const alertSchema = joi_1.default.alternatives().try(
    // Legacy alerts
    joi_1.default.string().custom((alert) => {
        if (alert === ALERT_QUERY) {
            return {
                id: (0, uuid_1.v4)(),
                assertion: 'response.status < 200 or response.status > 299',
                message: 'HTTP status is {{ response.status }}, expecting 2xx',
            };
        }
        if (alert.startsWith(RESPONSE_TIME_PREFIX)) {
            const expectedTime = parseAlertStringTime(alert);
            return {
                id: (0, uuid_1.v4)(),
                assertion: `response.time > ${expectedTime}`,
                message: `Response time is {{ response.time }}ms, expecting less than ${expectedTime}ms`,
            };
        }
        throw new Error(`Probe alert format is invalid! (${alert})`);
    }), joi_1.default
        .object({
        assertion: joi_1.default
            .string()
            .allow('')
            .default((parent) => {
            if (!parent.query) {
                return 'response.status < 200 or response.status > 299';
            }
            return parent.query;
        }),
        id: joi_1.default.string().allow(''),
        message: joi_1.default.string().allow(''),
        query: joi_1.default.string().allow(''),
    })
        .custom((alert) => {
        try {
            isValidProbeAlert(alert);
        }
        catch (error) {
            if ((0, __1.isSymonModeFrom)((0, context_1.getContext)().flags)) {
                pino_1.log.error(error?.message);
                return '';
            }
            throw new Error(`Probe alert format is invalid! (${JSON.stringify(alert, null, 1)})`);
        }
        return alert;
    }));
    const mysqlSchema = joi_1.default.object({
        command: joi_1.default.string().allow(''),
        data: joi_1.default.alternatives().try(joi_1.default.string().allow(''), joi_1.default.number()),
        database: joi_1.default.string().allow(''),
        host: joi_1.default
            .alternatives()
            .try(joi_1.default.string().allow('').hostname(), joi_1.default.string().allow('').ip())
            .required(),
        password: joi_1.default.string().allow(''),
        port: joi_1.default.number().default(3306),
        username: joi_1.default.string().allow(''),
    });
    const schema = joi_1.default.array().items(joi_1.default
        .object({
        alerts: joi_1.default
            .array()
            .items(alertSchema)
            .default((parent) => {
            if ((0, __1.isSymonModeFrom)((0, context_1.getContext)().flags)) {
                return [];
            }
            const isHTTPProbe = Boolean(parent.requests);
            if (!isHTTPProbe) {
                return [{ id: (0, uuid_1.v4)(), ...looper_1.FAILED_REQUEST_ASSERTION }];
            }
            return [
                {
                    id: (0, uuid_1.v4)(),
                    assertion: 'response.status < 200 or response.status > 299',
                    message: 'HTTP Status is {{ response.status }}, expecting 2xx',
                },
                {
                    id: (0, uuid_1.v4)(),
                    assertion: 'response.time > 2000',
                    message: 'Response time is {{ response.time }}ms, expecting less than 2000ms',
                },
                { id: (0, uuid_1.v4)(), ...looper_1.FAILED_REQUEST_ASSERTION },
            ];
        }),
        description: joi_1.default.string().allow(''),
        id: joi_1.default.string().required(),
        incidentThreshold: joi_1.default
            .number()
            .default(default_values_1.DEFAULT_INCIDENT_THRESHOLD)
            .min(1),
        recoveryThreshold: joi_1.default
            .number()
            .default(default_values_1.DEFAULT_RECOVERY_THRESHOLD)
            .min(1),
        interval: joi_1.default.number().default(default_values_1.DEFAULT_INTERVAL).min(1),
        lastEvent: joi_1.default.object({
            alertId: joi_1.default.string(),
            createdAt: joi_1.default.string().allow(''),
            recoveredAt: joi_1.default.string().allow('', null),
        }),
        name: joi_1.default
            .string()
            .allow('')
            .default((parent) => `monika_${parent.id}`),
        mariadb: joi_1.default.array().items(mysqlSchema),
        mongo: joi_1.default.array().items(joi_1.default.alternatives([
            joi_1.default.object({
                alerts: joi_1.default.array().items(alertSchema),
                uri: joi_1.default.string().required(),
            }),
            joi_1.default.object({
                alerts: joi_1.default.array().items(alertSchema),
                host: joi_1.default
                    .alternatives()
                    .try(joi_1.default.string().allow('').hostname(), joi_1.default.string().allow('').ip())
                    .required(),
                password: joi_1.default.string().allow(''),
                port: joi_1.default.number().default(27017).min(0).max(65536),
                username: joi_1.default.string().allow(''),
            }),
        ])),
        mysql: joi_1.default.array().items(mysqlSchema),
        ping: joi_1.default.array().items(joi_1.default.object({
            alerts: joi_1.default.array().items(alertSchema),
            uri: joi_1.default.string().required(),
        })),
        postgres: joi_1.default.array().items(joi_1.default.alternatives([
            joi_1.default.object({
                alerts: joi_1.default.array().items(alertSchema),
                uri: joi_1.default.string().required(),
            }),
            joi_1.default.object({
                alerts: joi_1.default.array().items(alertSchema),
                command: joi_1.default.string().allow(''),
                data: joi_1.default
                    .alternatives()
                    .try(joi_1.default.string().allow(''), joi_1.default.number()),
                database: joi_1.default.string().allow(''),
                host: joi_1.default
                    .alternatives()
                    .try(joi_1.default.string().allow('').hostname(), joi_1.default.string().allow('').ip())
                    .required(),
                password: joi_1.default.string().allow(''),
                port: joi_1.default.number().default(5432),
                username: joi_1.default.string().allow(''),
            }),
        ])),
        redis: joi_1.default.array().items(joi_1.default
            .object({
            alerts: joi_1.default.array().items(alertSchema),
            command: joi_1.default.string().allow(''),
            host: joi_1.default
                .alternatives()
                .try(joi_1.default.string().allow('').hostname(), joi_1.default.string().allow('').ip()),
            password: joi_1.default.string().allow(''),
            port: joi_1.default.number().min(0).max(65536),
            uri: joi_1.default.string().allow(''),
            username: joi_1.default.string().allow(''),
        })
            .xor('host', 'uri')
            .and('host', 'port')),
        requests: joi_1.default
            .array()
            .min((0, __1.isSymonModeFrom)((0, context_1.getContext)().flags) ? 0 : 1)
            .items(joi_1.default.object({
            alerts: joi_1.default.array().items(alertSchema),
            allowUnauthorized: joi_1.default.bool(),
            body: joi_1.default
                .alternatives()
                .try(joi_1.default.string().allow('', null), joi_1.default.object(), joi_1.default.array(), joi_1.default.number(), joi_1.default.bool()),
            followRedirects: joi_1.default
                .number()
                .min(0)
                .default((0, context_1.getContext)().flags['follow-redirects']),
            headers: joi_1.default.object().allow(null),
            id: joi_1.default.string().allow(''),
            interval: joi_1.default.number().min(1),
            method: joi_1.default
                .string()
                .valid('CONNECT', 'GET', 'POST', 'PUT', 'PATCH', 'DELETE', 'HEAD', 'OPTIONS', 'PURGE', 'LINK', 'TRACE', 'UNLINK')
                .default('GET')
                .insensitive()
                .label('Probe request method'),
            ping: joi_1.default.bool(),
            saveBody: joi_1.default.bool().default(false),
            timeout: joi_1.default.number().default(10000).min(1).allow(null),
            url: joi_1.default
                .string()
                .custom((url) => {
                if (!(0, is_valid_url_1.isValidURL)(url)) {
                    throw new Error(`Probe request URL (${url}) should start with http:// or https://`);
                }
                return url;
            })
                .label('Probe request URL')
                .required(),
        }))
            .label('Probe requests'),
        socket: joi_1.default.object({
            alerts: joi_1.default.array().items(alertSchema),
            data: joi_1.default.string().allow('', null),
            host: joi_1.default.string().required(),
            port: joi_1.default.number().required(),
        }),
    })
        .custom((probe) => {
        if (!isProbeRequestExists(probe)) {
            throw new Error('Probe requests does not exists or has length lower than 1!');
        }
        return probe;
    }));
    try {
        return await schema.validateAsync(probes, {
            stripUnknown: true,
        });
    }
    catch (error) {
        throw new Error(`Monika configuration is invalid. Probe: ${(0, catch_error_handler_1.getErrorMessage)(error)}`);
    }
}
exports.validateProbes = validateProbes;
function isProbeRequestExists(probe) {
    return (probe?.mariadb ||
        probe?.mongo ||
        probe?.mysql ||
        probe?.ping ||
        probe?.postgres ||
        probe?.redis ||
        probe?.requests ||
        probe?.socket);
}
// parse string like "response-time-greater-than-200-ms" and return the time in ms
function parseAlertStringTime(str) {
    // match any string that ends with digits followed by unit 's' or 'ms'
    const match = str.match(/(\d+)-(m?s)$/);
    if (!match) {
        throw new Error('Alert string does not contain valid time number');
    }
    const number = Number(match[1]);
    const unit = match[2];
    if (unit === 's')
        return number * 1000;
    return number;
}
function isValidProbeAlert(alert) {
    const expression = alert.assertion || alert.query;
    if (!expression) {
        return;
    }
    const data = {
        response: {
            size: 0,
            status: 200,
            time: 0,
            body: '',
            headers: {},
        },
    };
    const filter = (0, expression_parser_1.compileExpression)(expression, Object.keys(data));
    filter(data);
}
//# sourceMappingURL=probe.js.map