"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_fs_1 = require("node:fs");
const promises_1 = require("node:fs/promises");
const chai_1 = require("chai");
const msw_1 = require("msw");
const node_1 = require("msw/node");
const context_1 = require("../../context");
const flag_1 = require("../../flag");
const probe_1 = require("./probe");
const watcher_1 = require("./watcher");
describe('Config watcher', () => {
    const config = {
        probes: [
            {
                id: '1',
                requests: [{ url: 'https://example.com' }],
            },
        ],
    };
    const server = (0, node_1.setupServer)(msw_1.http.get('https://example.com/monika.json', () => new msw_1.HttpResponse(JSON.stringify(config))));
    before(() => {
        server.listen();
    });
    afterEach(() => {
        server.resetHandlers();
        (0, context_1.resetContext)();
    });
    after(() => {
        server.close();
    });
    it('should polling config from a URL', async () => {
        // arrange
        const configIntervalSeconds = 1;
        (0, context_1.setContext)({
            flags: (0, flag_1.sanitizeFlags)({
                config: ['https://example.com/monika.json'],
                'config-interval': configIntervalSeconds,
            }),
        });
        // act
        const watchers = (0, watcher_1.watchConfigChanges)();
        const seconds = 1000;
        await sleep(configIntervalSeconds * seconds);
        // assert
        (0, chai_1.expect)((0, probe_1.getProbes)()[0].requests?.[0].url).eq(config.probes[0].requests?.[0].url);
        for (const { cancel } of watchers) {
            cancel();
        }
    });
    it('should watch config file changes', async () => {
        // arrange
        if ((0, node_fs_1.existsSync)('monika.json')) {
            await (0, promises_1.rename)('monika.json', 'monika_backup.json');
        }
        await (0, promises_1.writeFile)('monika.json', JSON.stringify(config), { encoding: 'utf8' });
        (0, context_1.setContext)({
            flags: (0, flag_1.sanitizeFlags)({
                config: ['monika.json'],
            }),
        });
        // act
        const watchers = (0, watcher_1.watchConfigChanges)();
        const newConfig = {
            probes: [
                {
                    id: '2',
                    requests: [
                        {
                            url: 'https://example.com/changed',
                        },
                    ],
                },
            ],
        };
        await (0, promises_1.writeFile)('monika.json', JSON.stringify(newConfig), {
            encoding: 'utf8',
        });
        await sleep(1000);
        // assert
        (0, chai_1.expect)((0, probe_1.getProbes)()[0].id).eq('2');
        (0, chai_1.expect)((0, probe_1.getProbes)()[0].requests?.[0].url).eq('https://example.com/changed');
        for (const { cancel } of watchers) {
            cancel();
        }
        if ((0, node_fs_1.existsSync)('monika_backup.json')) {
            await (0, promises_1.rename)('monika_backup.json', 'monika.json');
        }
    });
});
function sleep(durationMs) {
    return new Promise((resolve) => {
        setTimeout(resolve, durationMs);
    });
}
//# sourceMappingURL=watcher.test.js.map