"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.closeLog = exports.getSummary = exports.saveNotificationLog = exports.saveProbeRequestLog = exports.flushAllLogs = exports.deleteNotificationLogs = exports.deleteRequestLogs = exports.getUnreportedLogs = exports.getUnreportedLogsCount = exports.getAllLogs = exports.deleteFromNotifications = exports.deleteFromAlerts = exports.deleteFromProbeRequests = exports.openLogfile = exports.setDatabase = exports.database = void 0;
const path_1 = __importDefault(require("path"));
const sqlite3_1 = require("sqlite3");
const sqlite_1 = require("sqlite");
const pino_1 = require("../../utils/pino");
const catch_error_handler_1 = require("../../utils/catch-error-handler");
const compact_config_1 = require("../config/compact-config");
const sqlite3 = (0, sqlite3_1.verbose)();
const dbPath = path_1.default.resolve(process.cwd(), 'monika-logs.db');
let db;
function database() {
    return db;
}
exports.database = database;
function setDatabase(database) {
    db = database;
}
exports.setDatabase = setDatabase;
async function migrate() {
    await database().migrate({
        // eslint-disable-next-line unicorn/prefer-module
        migrationsPath: path_1.default.join(__dirname, '../../../db/migrations'),
    });
}
/**
 * openLogfile will open the file history.db and if it doesn't exist, create it and sets up the table
 * @returns Promise<void>
 */
async function openLogfile() {
    try {
        const db = await (0, sqlite_1.open)({
            filename: dbPath,
            // eslint-disable-next-line no-bitwise
            mode: sqlite3.OPEN_READWRITE | sqlite3.OPEN_CREATE,
            driver: sqlite3.Database,
        });
        setDatabase(db);
        await migrate();
    }
    catch (error) {
        pino_1.log.error("Warning: Can't open logfile. " + (0, catch_error_handler_1.getErrorMessage)(error));
    }
}
exports.openLogfile = openLogfile;
async function deleteFromProbeRequests(limit) {
    const getIdsToBeDeleted = `SELECT id, probe_id, created_at FROM probe_requests order by created_at asc limit ${limit}`;
    const idsres = await database().all(getIdsToBeDeleted);
    const ids = idsres.map((res) => ({ id: res.id, createdAt: res.createdAt }));
    const probeIds = idsres.map((res) => ({
        id: res.probe_id,
        createdAt: res.created_at,
    }));
    if (idsres.length > 0) {
        const deleteFromProbeRequests = `DELETE FROM probe_requests WHERE id IN (${getIdsToBeDeleted})`;
        await database().run(deleteFromProbeRequests);
    }
    return {
        probeIds,
        probeRequestIds: ids,
    };
}
exports.deleteFromProbeRequests = deleteFromProbeRequests;
async function deleteFromAlerts(probeReqIds) {
    if (probeReqIds.length > 0) {
        await Promise.all(probeReqIds.map(async (item) => {
            const deleteFromAlerts = `DELETE FROM alerts WHERE probe_request_id = ${item.id} and created_at = ${item.createdAt}`;
            await database().run(deleteFromAlerts);
        }));
    }
}
exports.deleteFromAlerts = deleteFromAlerts;
async function deleteFromNotifications(probeIds) {
    if (probeIds.length > 0) {
        await Promise.all(probeIds.map(async (item) => {
            const deleteFromNotifications = `DELETE FROM notifications WHERE probe_id = ${item.id} and created_at = ${item.createdAt}`;
            await database().run(deleteFromNotifications);
        }));
    }
}
exports.deleteFromNotifications = deleteFromNotifications;
/**
 * getAllLogs gets all the history table from sqlite db
 * @returns {obj} result of logs table
 */
async function getAllLogs() {
    const readRowsSQL = 'SELECT id, probe_id, response_status, request_url, response_time FROM probe_requests';
    const probeRequests = await database().all(readRowsSQL);
    const dbVal = probeRequests.map((probeRequest) => {
        /* eslint-disable camelcase */
        const { id, probe_id, request_url, response_status, response_time } = probeRequest;
        return {
            id,
            probeId: probe_id,
            responseStatus: response_status,
            requestUrl: request_url,
            responseTime: response_time,
        };
        /* eslint-enable camelcase */
    });
    return dbVal;
}
exports.getAllLogs = getAllLogs;
async function getUnreportedLogsCount() {
    const readUnreportedRequestsCountSQL = `
    SELECT COUNT(id) as count
    FROM probe_requests
    WHERE reported = 0;`;
    const row = await database().get(readUnreportedRequestsCountSQL);
    return row?.count || 0;
}
exports.getUnreportedLogsCount = getUnreportedLogsCount;
async function getUnreportedLogs(ids, limit, database = db) {
    const readUnreportedRequestsSQL = `
    SELECT PR.id,
      PR.created_at as timestamp,
      PR.probe_id,
      PR.response_status,
      PR.response_time,
      CASE
        WHEN A.type IS NULL THEN json_array()
        ELSE json_group_array(A.type)
      END alerts
    FROM probe_requests PR
      LEFT JOIN alerts A ON PR.id = A.probe_request_id
    WHERE PR.reported = 0 AND PR.probe_id IN ('${ids.join("','")}')
    GROUP BY PR.id
    ORDER BY PR.created_at ASC
    LIMIT ${limit};`;
    const readUnreportedNotificationsSQL = `
    SELECT id,
      created_at as timestamp,
      probe_id,
      probe_name,
      alert_type,
      type,
      notification_id,
      channel
    FROM notifications
    WHERE reported = 0 AND probe_id IN ('${ids.join("','")}')
    ORDER BY timestamp ASC
    LIMIT ${limit};`;
    const [unreportedRequests, unreportedNotifications] = await Promise.all([
        database.all(readUnreportedRequestsSQL).then((unreportedProbeRequests) => 
        /* eslint-disable camelcase */
        unreportedProbeRequests.map((unreportedProbeRequest) => {
            const { alerts, id, probe_id, response_status, response_time, timestamp, } = unreportedProbeRequest;
            return {
                alerts: JSON.parse(alerts),
                id,
                probeId: probe_id,
                responseStatus: response_status,
                responseTime: response_time,
                timestamp,
            };
        })
        /* eslint-enable camelcase */
        ),
        database
            .all(readUnreportedNotificationsSQL)
            .then((unreportedNotifications) => unreportedNotifications.map((unreportedNotification) => {
            /* eslint-disable camelcase */
            const { alert_type, channel, id, notification_id, probe_id, probe_name, timestamp, type, } = unreportedNotification;
            return {
                alertType: alert_type,
                channel,
                id,
                notificationId: notification_id,
                probeId: probe_id,
                probeName: probe_name,
                timestamp,
                type,
            };
            /* eslint-enable camelcase */
        })),
    ]);
    return {
        requests: unreportedRequests,
        notifications: unreportedNotifications,
    };
}
exports.getUnreportedLogs = getUnreportedLogs;
async function deleteRequestLogs(ids, database = db) {
    const idsString = ids.join("','");
    const sql = `DELETE FROM probe_requests WHERE probe_id IN ('${idsString}');`;
    return database.run(sql);
}
exports.deleteRequestLogs = deleteRequestLogs;
async function deleteNotificationLogs(ids, database = db) {
    const idsString = ids.join("','");
    const sql = `DELETE FROM notifications WHERE probe_id IN ('${idsString}');`;
    return database.run(sql);
}
exports.deleteNotificationLogs = deleteNotificationLogs;
/**
 * flushAllLogs drops the table and recreates it
 * @returns Promise<void>
 */
async function flushAllLogs() {
    const dropAtlassianStatusPageTableSQL = 'DROP TABLE IF EXISTS atlassian_status_page_incidents;';
    const dropInstatusPageTableSQL = 'DROP TABLE IF EXISTS instatus_page_incidents;';
    const dropProbeRequestsTableSQL = 'DROP TABLE IF EXISTS probe_requests;';
    const dropAlertsTableSQL = 'DROP TABLE IF EXISTS alerts;';
    const dropNotificationsTableSQL = 'DROP TABLE IF EXISTS notifications;';
    const dropMigrationsTableSQL = 'DROP TABLE IF EXISTS migrations;';
    await Promise.all([
        database().run(dropAtlassianStatusPageTableSQL),
        database().run(dropProbeRequestsTableSQL),
        database().run(dropAlertsTableSQL),
        database().run(dropNotificationsTableSQL),
        database().run(dropMigrationsTableSQL),
        database().run(dropInstatusPageTableSQL),
        // The VACUUM command cleans the main database by copying its contents to a temporary database file and reloading the original database file from the copy.
        // This eliminates free pages, aligns table data to be contiguous, and otherwise cleans up the database file structure.
        // When VACUUMing a database, as much as twice the size of the original database file is required in free disk space.
        database().run('vacuum'),
    ]);
    await migrate();
}
exports.flushAllLogs = flushAllLogs;
/**
 * saveProbeRequestLog inserts probe request log information into the database
 *
 * @param {object} data is the log data containing information about probe request
 * @returns Promise<void>
 */
async function saveProbeRequestLog({ probe, requestIndex, probeRes, alertQueries, error: errorResp, }) {
    if (probe.jointId?.length) {
        const probes = probe.jointId
            ?.map((id) => (0, compact_config_1.getDecompactedProbesById)(id))
            ?.filter(Boolean) || [];
        await Promise.allSettled(probes.map((originalProbe) => saveProbeRequestLog({
            probe: originalProbe,
            requestIndex,
            probeRes,
            alertQueries: originalProbe.alerts
                ?.map((alert) => alert.query)
                .filter(Boolean) || [],
            error: errorResp,
        })));
        return;
    }
    const insertProbeRequestSQL = `
    INSERT INTO probe_requests (
        created_at,
        probe_id,
        probe_name,
        request_method,
        request_url,
        request_header,
        request_body,
        response_status,
        response_header,
        response_body,
        response_time,
        response_size,
        error,
        request_type,
        socket_host,
        socket_port
      )
    VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);`;
    const insertAlertSQL = `
    INSERT INTO alerts (
        created_at,
        probe_request_id,
        type
      )
    VALUES (?, ?, ?);`;
    const now = Math.round(Date.now() / 1000);
    const requestConfig = probe.requests?.[requestIndex];
    const responseBody = requestConfig?.saveBody
        ? typeof probeRes.data === 'string'
            ? probeRes.data
            : JSON.stringify(probeRes.data)
        : '';
    try {
        const insertProbeRequestResult = await database().run(insertProbeRequestSQL, [
            now,
            probe.id,
            probe.name,
            requestConfig?.method || 'TCP',
            requestConfig?.url || 'http://',
            JSON.stringify(requestConfig?.headers),
            JSON.stringify(requestConfig?.body),
            probeRes.status,
            JSON.stringify(probeRes.headers),
            responseBody,
            probeRes?.responseTime ?? 0,
            typeof probeRes.headers === 'string'
                ? probeRes.headers
                : probeRes.headers['content-length'],
            errorResp,
            probe.socket ? 'tcp' : 'http',
            probe.socket?.host || '',
            probe.socket?.port || '',
        ]);
        await Promise.all((alertQueries ?? []).map((alert) => database().run(insertAlertSQL, [
            now,
            insertProbeRequestResult.lastID,
            alert,
        ])));
    }
    catch (error) {
        pino_1.log.error("Error: Can't insert data into monika-log.db. " + (0, catch_error_handler_1.getErrorMessage)(error));
    }
}
exports.saveProbeRequestLog = saveProbeRequestLog;
/**
 * saveNotificationLog inserts probe request log information into the database
 *
 * @param {object} probe is the probe config
 * @param {object} notification is the notification config
 * @param {string} type is the type of notification 'NOTIFY-INCIDENT' | 'NOTIFY-RECOVER'
 * @param {string} alertQuery the alerts triggered
 * @returns Promise<void>
 */
async function saveNotificationLog(probe, notification, type, alertQuery) {
    const insertNotificationSQL = `
    INSERT INTO notifications (
        created_at,
        probe_id,
        probe_name,
        alert_type,
        type,
        notification_id,
        channel
      )
    VALUES (?, ?, ?, ?, ?, ?, ?);`;
    const now = Math.round(Date.now() / 1000);
    try {
        await database().run(insertNotificationSQL, [
            now,
            probe.id,
            probe.name,
            alertQuery,
            type,
            notification.id,
            notification.type,
        ]);
    }
    catch (error) {
        pino_1.log.error("Error: Can't insert data into monika-log.db. " + (0, catch_error_handler_1.getErrorMessage)(error));
    }
}
exports.saveNotificationLog = saveNotificationLog;
async function getSummary() {
    const getNotificationsSummaryByTypeSQL = `SELECT type, COUNT(*) as count FROM notifications WHERE created_at > strftime('%s', datetime('now', '-24 hours')) GROUP BY type;`;
    const notificationsSummaryByType = await database().all(getNotificationsSummaryByTypeSQL);
    const numberOfIncidents = notificationsSummaryByType.find((notif) => notif.type === 'NOTIFY-INCIDENT')
        ?.count || 0;
    const numberOfRecoveries = notificationsSummaryByType.find((notif) => notif.type === 'NOTIFY-RECOVER')
        ?.count || 0;
    const numberOfSentNotifications = notificationsSummaryByType.reduce((acc, { count }) => acc + count, 0);
    return {
        numberOfIncidents,
        numberOfRecoveries,
        numberOfSentNotifications,
    };
}
exports.getSummary = getSummary;
/**
 * closeLog closes the database
 * @returns Promise<void>
 */
async function closeLog() {
    await db?.close();
}
exports.closeLog = closeLog;
//# sourceMappingURL=history.js.map