"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
exports.doProbe = void 0;
const cockatiel_1 = require("cockatiel");
const date_fns_1 = require("date-fns");
const context_1 = require("../../context");
const probe_state_1 = require("../../utils/probe-state");
const factory_1 = require("./prober/factory");
const default_values_1 = require("../config/validation/validator/default-values");
/**
 * doProbe sends out the http request
 * @param {object} doProbeParams doProbe parameter
 * @returns {Promise<void>} void
 */
async function doProbe({ probe, notifications, signal, }) {
    if (!isTimeToProbe(probe) || isCycleEnd(probe.id)) {
        return;
    }
    const randomTimeoutMilliseconds = getRandomTimeoutMilliseconds();
    (0, probe_state_1.setProbeRunning)(probe.id);
    const randomTimeoutInterval = setTimeout(async () => {
        const probeCtx = (0, probe_state_1.getProbeContext)(probe.id);
        if (!probeCtx || signal.aborted) {
            clearTimeout(randomTimeoutInterval);
            return;
        }
        const probers = (0, factory_1.createProbers)({
            counter: probeCtx.cycle,
            notifications,
            probeConfig: probe,
        });
        const maxAttempts = Math.max(
        // since we will retry for both incident and recovery, let's just get the biggest threshold
        probe.incidentThreshold || default_values_1.DEFAULT_INCIDENT_THRESHOLD, probe.recoveryThreshold || default_values_1.DEFAULT_RECOVERY_THRESHOLD);
        await (0, cockatiel_1.retry)(cockatiel_1.handleAll, {
            maxAttempts,
            backoff: new cockatiel_1.ExponentialBackoff({
                initialDelay: (0, context_1.getContext)().flags.retryInitialDelayMs,
                maxDelay: (0, context_1.getContext)().flags.retryMaxDelayMs,
            }),
        }).execute(({ attempt }) => Promise.all(probers.map(async (prober) => prober.probe({ incidentRetryAttempt: attempt, signal }))));
        if (!signal.aborted) {
            (0, probe_state_1.setProbeFinish)(probe.id);
        }
    }, randomTimeoutMilliseconds);
}
exports.doProbe = doProbe;
function isTimeToProbe({ id, interval }) {
    const probeCtx = (0, probe_state_1.getProbeContext)(id);
    if (!probeCtx) {
        return false;
    }
    const isIdle = (0, probe_state_1.getProbeState)(id) === 'idle';
    const isInTime = (0, date_fns_1.differenceInSeconds)(new Date(), probeCtx.lastFinish) >= interval;
    return isIdle && isInTime;
}
function isCycleEnd(probeID) {
    const probeCtx = (0, probe_state_1.getProbeContext)(probeID);
    if (!probeCtx) {
        return true;
    }
    return ((0, context_1.getContext)().flags.repeat && (0, context_1.getContext)().flags.repeat === probeCtx.cycle);
}
function getRandomTimeoutMilliseconds() {
    return [1000, 2000, 3000].sort(() => Math.random() - 0.5)[0];
}
//# sourceMappingURL=index.js.map