"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HTTPProber = void 0;
const __1 = require("..");
const context_1 = require("../../../../context");
const events_1 = __importDefault(require("../../../../events"));
const request_1 = require("../../../../interfaces/request");
const checkers_1 = __importDefault(require("../../../../plugins/validate-response/checkers"));
const alert_id_1 = require("../../../../utils/alert-id");
const events_2 = require("../../../../utils/events");
const config_1 = require("../../../config");
const incident_1 = require("../../../incident");
const history_1 = require("../../../logger/history");
const response_time_log_1 = require("../../../logger/response-time-log");
const request_2 = require("./request");
const response_cache_1 = require("./response-cache");
class HTTPProber extends __1.BaseProber {
    async probe({ incidentRetryAttempt, signal }) {
        const requests = this.probeConfig.requests;
        // sending multiple http requests for request chaining
        const responses = [];
        // do http request
        // force fresh request if :
        // - probe has chaining requests, OR
        // - this is a retrying attempt
        if (requests.length > 1 ||
            incidentRetryAttempt > 0 ||
            !(0, context_1.getContext)().flags['ttl-cache']) {
            for (const requestConfig of requests) {
                responses.push(
                // eslint-disable-next-line no-await-in-loop
                await this.doRequest(requestConfig, signal, responses));
            }
        }
        else {
            // use cached response when possible
            // or fallback to fresh request if cache expired
            const responseCache = (0, response_cache_1.get)(requests[0]);
            const response = responseCache || (await this.doRequest(requests[0], signal, responses));
            if (!responseCache)
                (0, response_cache_1.put)(requests[0], response);
            responses.push(response);
        }
        const hasFailedRequest = responses.find(({ result }) => result !== request_1.probeRequestResult.success);
        if (hasFailedRequest) {
            if (this.hasIncident()) {
                // this probe is currently 'incident' state so no need to continue
                this.logMessage(false, getErrorMessage(hasFailedRequest.error || 'Unknown error.'));
                return;
            }
            // if the incident threshold is not yet met, this will throw and return the execution to `retry` function in src/components/probe/index.ts
            this.throwIncidentIfNeeded(incidentRetryAttempt, this.probeConfig.incidentThreshold);
            // the threshold has been met, so let's log the message
            this.logMessage(false, getErrorMessage(hasFailedRequest.error || 'Unknown error.'), getNotificationMessage({ isIncident: true }));
            // this probe is definitely in incident state, so send notification, etc.
            this.handleFailedProbe(responses.map((requestResponse) => ({ requestResponse })));
            return;
        }
        // from here on, the probe can be accessed but might still trigger the assertion
        for (const requestIndex of responses.keys()) {
            const response = responses[requestIndex];
            const validatedResponse = this.validateResponse(response, requests[requestIndex].alerts || []);
            const triggeredAlertResponse = validatedResponse.find(({ isAlertTriggered }) => isAlertTriggered);
            if (triggeredAlertResponse) {
                const { alert } = triggeredAlertResponse;
                if (this.hasIncident()) {
                    // this probe is still in incident state
                    this.logMessage(false, getAssertionMessage(alert.assertion));
                    return;
                }
                // if the incident threshold is not yet met, this will throw and return the execution to `retry` function in src/components/probe/index.ts
                this.throwIncidentIfNeeded(incidentRetryAttempt, this.probeConfig.incidentThreshold, 'Probe assertion failed');
                // this probe is definitely in incident state because of fail assertion, so send notification, etc.
                this.handleAssertionFailed(response, requestIndex, alert);
                return;
            }
        }
        // from here on, the probe is definitely healthy, but if it was incident, we don't want to immediately send notification
        this.sendRecoveryNotificationIfNeeded(incidentRetryAttempt, responses.map((requestResponse) => ({ requestResponse })));
        // the probe is healthy and not recovery
        for (const requestIndex of responses.keys()) {
            const response = responses[requestIndex];
            (0, events_2.getEventEmitter)().emit(events_1.default.probe.response.received, {
                probe: this.probeConfig,
                requestIndex,
                response,
            });
            (0, events_2.getEventEmitter)().emit(events_1.default.probe.status.changed, {
                probe: this.probeConfig,
                requestIndex,
                status: 'up',
            });
            this.logMessage(true, getProbeResultMessage({
                request: requests[requestIndex],
                response,
            }));
            (0, response_time_log_1.logResponseTime)(response.responseTime);
            if ((0, config_1.isSymonModeFrom)((0, context_1.getContext)().flags) ||
                (0, context_1.getContext)().flags['keep-verbose-logs']) {
                (0, history_1.saveProbeRequestLog)({
                    probe: this.probeConfig,
                    requestIndex,
                    probeRes: response,
                });
            }
        }
    }
    doRequest(config, signal, responses) {
        return (0, request_2.httpRequest)({
            requestConfig: {
                ...config,
                allowUnauthorized: config.allowUnauthorized ?? (0, context_1.getContext)().flags.ignoreInvalidTLS,
                signal,
            },
            responses,
        });
    }
    generateVerboseStartupMessage() {
        const { description, id, interval, name } = this.probeConfig;
        let result = `- Probe ID: ${id}
  Name: ${name}
  Description: ${description || '-'}
  Interval: ${interval}
`;
        result += '  Requests:\n';
        result += this.generateProbeRequestMessage();
        result += this.generateAlertMessage();
        return result;
    }
    handleAssertionFailed(response, requestIndex, triggeredAlert) {
        const probeID = this.probeConfig.id;
        const url = this.probeConfig?.requests?.[requestIndex].url || '';
        const validation = {
            alert: triggeredAlert,
            isAlertTriggered: true,
            response,
        };
        const alertId = (0, alert_id_1.getAlertID)(url, validation, probeID);
        (0, events_2.getEventEmitter)().emit(events_1.default.probe.status.changed, {
            probe: this.probeConfig,
            requestIndex,
            status: 'down',
        });
        (0, events_2.getEventEmitter)().emit(events_1.default.probe.alert.triggered, {
            probe: this.probeConfig,
            requestIndex,
            alertQuery: triggeredAlert,
        });
        (0, incident_1.addIncident)({
            alert: triggeredAlert,
            probeID,
            probeRequestURL: url,
        });
        this.sendNotification({
            requestURL: url,
            notificationType: __1.NotificationType.Incident,
            validation,
            alertId,
        });
        this.logMessage(false, getAssertionMessage(triggeredAlert.assertion), getNotificationMessage({ isIncident: true }));
    }
    generateProbeRequestMessage() {
        let startupMessage = '';
        for (const request of this.probeConfig.requests || []) {
            const { body, headers, method, url } = request;
            startupMessage += `  - Request Method: ${method || `GET`}
    Request URL: ${url}
    Request Headers: ${JSON.stringify(headers) || `-`}
    Request Body: ${JSON.stringify(body) || `-`}
`;
        }
        return startupMessage;
    }
    generateAlertMessage() {
        const alertsInString = JSON.stringify(this.probeConfig.alerts);
        return `    Alerts: ${alertsInString}\n`;
    }
    validateResponse(response, additionalAssertions) {
        const assertions = [...this.probeConfig.alerts, ...additionalAssertions];
        return assertions.map((assertion) => ({
            alert: assertion,
            isAlertTriggered: (0, checkers_1.default)(assertion, response),
            response,
        }));
    }
}
exports.HTTPProber = HTTPProber;
function getProbeResultMessage({ request, response, }) {
    if (request?.ping) {
        return response?.body;
    }
    if ((0, context_1.getContext)().flags.verbose) {
        return `${response?.status || '-'} ${request?.method} ${request?.url} ${response?.responseTime || '-'}ms
    Request Headers: ${JSON.stringify(request?.headers) || '-'}
    Request Body: ${JSON.stringify(request?.body) || '-'}
    Response Body: ${JSON.stringify(response?.body) || '-'}`;
    }
    return `${response?.status || '-'} ${request?.method} ${request?.url} ${response?.responseTime || '-'}ms`;
}
function getErrorMessage(message) {
    return `ERROR: ${message}`;
}
function getAssertionMessage(message) {
    return `ASSERTION: ${message}`;
}
function getNotificationMessage({ isIncident, }) {
    return `NOTIF: ${isIncident ? 'Service probably down' : 'Service is back up'}`;
}
//# sourceMappingURL=index.js.map