"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_1 = require("@oclif/test");
const msw_1 = require("msw");
const node_1 = require("msw/node");
const sinon_1 = __importDefault(require("sinon"));
const __1 = require("../..");
const probe_state_1 = require("../../../../utils/probe-state");
const context_1 = require("../../../../context");
const looper_1 = require("../../../../looper");
const history_1 = require("../../../logger/history");
let urlRequestTotal = 0;
let notificationAlert = {};
const server = (0, node_1.setupServer)(msw_1.http.get('https://example.com', () => {
    urlRequestTotal += 1;
    return new msw_1.HttpResponse(null, {
        status: 200,
    });
}), msw_1.http.post('https://example.com/webhook', async ({ request }) => {
    const requestBody = (await request.json());
    if (requestBody?.body?.url) {
        notificationAlert[requestBody.body.url] = requestBody;
    }
    return new msw_1.HttpResponse(null, {
        status: 200,
    });
}));
const probes = [
    {
        id: '1',
        name: 'Example',
        interval: 1,
        requests: [
            {
                url: 'https://example.com',
                body: '',
                followRedirects: 21,
                timeout: 30,
            },
        ],
        alerts: [],
    },
];
describe('HTTP Probe processing', () => {
    before(async () => {
        server.listen();
        await (0, history_1.openLogfile)();
    });
    beforeEach(() => {
        (0, context_1.setContext)({ flags: { repeat: 1 } });
    });
    afterEach(() => {
        (0, context_1.resetContext)();
        urlRequestTotal = 0;
        notificationAlert = {};
        server.resetHandlers();
        sinon_1.default.restore();
    });
    after(async () => {
        server.close();
        await (0, history_1.closeLog)();
    });
    it('should not run probe if the probe is running', async () => {
        // arrange
        (0, probe_state_1.initializeProbeStates)(probes);
        // wait until the interval passed
        const seconds = 1000;
        await sleep(seconds);
        // act
        (0, __1.doProbe)({
            probe: probes[0],
            notifications: [],
            signal: new AbortController().signal,
        });
        await (0, __1.doProbe)({
            probe: probes[0],
            notifications: [],
            signal: new AbortController().signal,
        });
        // wait for random timeout
        await sleep(3 * seconds);
        // assert
        (0, test_1.expect)(urlRequestTotal).eq(1);
    });
    it('should not run probe if it is not the time', () => {
        // arrange
        (0, probe_state_1.initializeProbeStates)(probes);
        // act
        (0, __1.doProbe)({
            notifications: [],
            probe: { ...probes[0], interval: 10 },
            signal: new AbortController().signal,
        });
        // assert
        (0, test_1.expect)(urlRequestTotal).eq(0);
    });
    it('should not run probe if the cycle is end', async () => {
        // arrange
        (0, probe_state_1.initializeProbeStates)(probes);
        (0, context_1.setContext)({
            ...(0, context_1.getContext)(),
            flags: { ...(0, context_1.getContext)().flags, repeat: 1 },
        });
        // wait until the interval passed
        const seconds = 1000;
        await sleep(seconds);
        // act
        await (0, __1.doProbe)({
            probe: probes[0],
            notifications: [],
            signal: new AbortController().signal,
        });
        await (0, __1.doProbe)({
            probe: probes[0],
            notifications: [],
            signal: new AbortController().signal,
        });
        await (0, __1.doProbe)({
            probe: probes[0],
            notifications: [],
            signal: new AbortController().signal,
        });
        // wait for random timeout
        await sleep(3 * seconds);
        // assert
        (0, test_1.expect)(urlRequestTotal).eq(1);
    });
    it('should run the probe', async () => {
        // arrange
        const uniqueProbes = Array.from({ length: 5 }).map((_, index) => ({
            ...probes[0],
            id: `${index}`,
        }));
        (0, probe_state_1.initializeProbeStates)(uniqueProbes);
        // wait until the interval passed
        const seconds = 1000;
        await sleep(seconds);
        // act
        await Promise.all(uniqueProbes.map((probe) => (0, __1.doProbe)({
            probe,
            notifications: [],
            signal: new AbortController().signal,
        })));
        // wait for random timeout
        await sleep(3 * seconds);
        // assert
        (0, test_1.expect)(urlRequestTotal).eq(5);
    });
    it('should send incident notification if the request is failed', async () => {
        // arrange
        server.use(msw_1.http.get('https://example.com', () => msw_1.HttpResponse.error()));
        const probe = {
            ...probes[0],
            id: '2md9a',
            requests: [
                {
                    url: 'https://example.com',
                    body: '',
                    followRedirects: 21,
                    timeout: 30,
                },
            ],
            alerts: [
                {
                    id: 'Cqkjh',
                    ...looper_1.FAILED_REQUEST_ASSERTION,
                },
                {
                    id: 'fKBzx',
                    assertion: 'response.status == 200',
                    message: 'The assertion failed.',
                },
            ],
        };
        (0, probe_state_1.initializeProbeStates)([probe]);
        // wait until the interval passed
        const seconds = 1000;
        await sleep(seconds);
        // act
        await (0, __1.doProbe)({
            probe,
            notifications: [
                {
                    id: 'jFQBd',
                    data: { url: 'https://example.com/webhook' },
                    type: 'webhook',
                },
            ],
            signal: new AbortController().signal,
        });
        // wait for random timeout
        await sleep(3 * seconds);
        // wait for send notification function to resolve
        await sleep(2 * seconds);
        // assert
        (0, test_1.expect)(notificationAlert?.[probe.requests[0].url]?.body?.url).eq('https://example.com');
        (0, test_1.expect)(notificationAlert?.[probe.requests[0].url]?.body.alert).eq('');
    }).timeout(10000);
    it('should send incident notification when assertion fails', async () => {
        // arrange
        server.use(msw_1.http.get('https://example.com', () => {
            urlRequestTotal += 1;
            return new msw_1.HttpResponse(null, {
                status: 404,
            });
        }));
        const probe = {
            ...probes[0],
            id: '2md9o',
            alerts: [
                {
                    id: 'P7-fN',
                    assertion: 'response.status != 200',
                    message: 'The assertion failed.',
                },
            ],
        };
        (0, probe_state_1.initializeProbeStates)([probe]);
        // wait until the interval passed
        const seconds = 1000;
        await sleep(seconds);
        // act
        await (0, __1.doProbe)({
            probe,
            notifications: [
                {
                    id: 'jFQBd',
                    data: { url: 'https://example.com/webhook' },
                    type: 'webhook',
                },
            ],
            signal: new AbortController().signal,
        });
        // wait for random timeout
        await sleep(3 * seconds);
        // wait for send notification function to resolve
        await sleep(2 * seconds);
        // assert
        (0, test_1.expect)(notificationAlert?.[probe?.requests?.[0]?.url || 0]?.body?.url).eq('https://example.com');
        (0, test_1.expect)(notificationAlert?.[probe?.requests?.[0]?.url || 0]?.body?.alert).eq('response.status != 200');
        // restore
        server.resetHandlers();
    }).timeout(10000);
    it('should send recovery notification', async () => {
        // arrange
        server.use(msw_1.http.get('https://example.com', () => {
            urlRequestTotal += 1;
            return new msw_1.HttpResponse(null, {
                status: 404,
            });
        }));
        const probe = {
            ...probes[0],
            id: 'fj43l',
            incidentThreshold: 1,
            requests: [
                {
                    url: 'https://example.com',
                    body: '',
                    followRedirects: 21,
                    timeout: 30,
                },
            ],
            alerts: [
                { id: 'jFQBd', assertion: 'response.status != 200', message: '' },
            ],
        };
        const notifications = [
            {
                id: 'jFQBd',
                data: { url: 'https://example.com/webhook' },
                type: 'webhook',
            },
        ];
        (0, probe_state_1.initializeProbeStates)([probe]);
        // wait until the interval passed
        const seconds = 1000;
        await sleep(seconds);
        // act
        await (0, __1.doProbe)({
            probe,
            notifications,
            signal: new AbortController().signal,
        });
        // wait for random timeout
        await sleep(3 * seconds);
        server.resetHandlers();
        // wait for the send notification function to resolve
        await sleep(3 * seconds);
        // assert
        (0, test_1.expect)(notificationAlert?.[probe.requests[0].url]?.body?.url).eq('https://example.com');
        (0, test_1.expect)(notificationAlert?.[probe.requests[0].url]?.body?.alert).eq('response.status != 200');
    }).timeout(10000);
});
function sleep(durationMs) {
    return new Promise((resolve) => {
        setTimeout(resolve, durationMs);
    });
}
//# sourceMappingURL=index.test.js.map