"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseProber = exports.NotificationType = void 0;
const context_1 = require("../../../context");
const events_1 = __importDefault(require("../../../events"));
const request_1 = require("../../../interfaces/request");
const looper_1 = require("../../../looper");
const alert_id_1 = require("../../../utils/alert-id");
const events_2 = require("../../../utils/events");
const pino_1 = require("../../../utils/pino");
const config_1 = require("../../config");
const default_values_1 = require("../../config/validation/validator/default-values");
const incident_1 = require("../../incident");
const history_1 = require("../../logger/history");
const response_time_log_1 = require("../../logger/response-time-log");
const notification_1 = require("../../notification");
var NotificationType;
(function (NotificationType) {
    NotificationType["Incident"] = "NOTIFY-INCIDENT";
    NotificationType["Recover"] = "NOTIFY-RECOVER";
})(NotificationType || (exports.NotificationType = NotificationType = {}));
var ProbeState;
(function (ProbeState) {
    ProbeState["Up"] = "UP";
    ProbeState["Down"] = "DOWN";
})(ProbeState || (ProbeState = {}));
class BaseProber {
    constructor({ counter, notifications, probeConfig }) {
        this.counter = counter;
        this.notifications = notifications;
        this.probeConfig = probeConfig;
        this.initializeProbeState();
    }
    processProbeResults(probeResults, incidentRetryAttempt) {
        for (const { isAlertTriggered, logMessage } of probeResults) {
            this.logMessage(!isAlertTriggered, logMessage);
        }
        if (probeResults.some(({ requestResponse }) => requestResponse.result !== request_1.probeRequestResult.success)) {
            if (this.hasIncident()) {
                // this probe is still in incident state
                return;
            }
            // if the incident threshold is not yet met, this will throw and return the execution to `retry` function in src/components/probe/index.ts
            this.throwIncidentIfNeeded(incidentRetryAttempt, this.probeConfig.incidentThreshold);
            // this probe is definitely in incident state because of fail assertion, so send notification, etc.
            this.handleFailedProbe(probeResults);
            return;
        }
        // from here on, the probe is definitely healthy, but if it was incident, we don't want to immediately send notification
        this.sendRecoveryNotificationIfNeeded(incidentRetryAttempt, probeResults);
        // the probe is healthy and not recovery
        for (const index of probeResults.keys()) {
            const { requestResponse } = probeResults[index];
            (0, events_2.getEventEmitter)().emit(events_1.default.probe.response.received, {
                probe: this.probeConfig,
                requestIndex: index,
                response: requestResponse,
            });
            (0, events_2.getEventEmitter)().emit(events_1.default.probe.status.changed, {
                probe: this.probeConfig,
                requestIndex: index,
                status: 'up',
            });
            (0, response_time_log_1.logResponseTime)(requestResponse.responseTime);
            if ((0, config_1.isSymonModeFrom)((0, context_1.getContext)().flags) ||
                (0, context_1.getContext)().flags['keep-verbose-logs']) {
                (0, history_1.saveProbeRequestLog)({
                    probe: this.probeConfig,
                    requestIndex: index,
                    probeRes: requestResponse,
                });
            }
        }
    }
    getFailedRequestAssertion(requestIndex) {
        const getFailedRequestAssertionFromProbe = () => this.probeConfig.alerts.find(({ assertion, message, query }) => (assertion === looper_1.FAILED_REQUEST_ASSERTION.assertion ||
            query === looper_1.FAILED_REQUEST_ASSERTION.assertion) &&
            message === looper_1.FAILED_REQUEST_ASSERTION.message);
        const getFailedRequestAssertionFromRequests = (requestIndex) => {
            if (this.probeConfig.requests === undefined ||
                requestIndex === undefined) {
                return undefined;
            }
            return (this.probeConfig.requests[requestIndex].alerts || []).find(({ assertion, message, query }) => (assertion === looper_1.FAILED_REQUEST_ASSERTION.assertion ||
                query === looper_1.FAILED_REQUEST_ASSERTION.assertion) &&
                message === looper_1.FAILED_REQUEST_ASSERTION.message);
        };
        return (getFailedRequestAssertionFromProbe() ||
            getFailedRequestAssertionFromRequests(requestIndex));
    }
    hasIncident() {
        return (0, incident_1.findIncident)(this.probeConfig.id);
    }
    /**
     * If the probe is healthy and previously not (so it's a recovery), this function will call the function to send recovery notification only when the retry attempts equals to the recovery threshold - 1.
     * Otherwise, it will throw and return the execution to the retry function in src/components/probe/index.ts.
     * If the probe is healthy just like before, nothing to do in this function.
     * @param incidentRetryAttempt The number of retry attempts
     * @param probeResults The probe results
     * @returns void
     */
    sendRecoveryNotificationIfNeeded(incidentRetryAttempt, probeResults) {
        const isRecoveryThresholdMet = incidentRetryAttempt ===
            (this.probeConfig.recoveryThreshold || default_values_1.DEFAULT_RECOVERY_THRESHOLD) - 1;
        const isRecovery = this.hasIncident();
        if (isRecovery) {
            if (!isRecoveryThresholdMet) {
                this.logMessage(false, `Probing succeeds but previously incident. Will retry. Attempt (${incidentRetryAttempt + 1}) with recovery threshold (${this.probeConfig.recoveryThreshold}).`);
                // throw here so that the retry function in src/components/probe/index.ts can retry again
                throw new Error('Probing succeeds but recovery threshold is not met.');
            }
            // at this state, the probe has definitely recovered, so send notifications, etc.
            this.handleRecovery(probeResults);
        }
    }
    /**
     * If the number of attempts is equal to the incidentThreshold - 1, this function will throw which will return execution to  the retry function in src/components/probe/index.ts.
     * Otherwise, it will not do anything.
     * @param incidentRetryAttempt How many times have monika retry probing
     * @param incidentThreshold The incident threshold of the probe
     * @param message Message to display to stdout
     * @throws
     * @returns void
     */
    throwIncidentIfNeeded(incidentRetryAttempt, incidentThreshold = default_values_1.DEFAULT_INCIDENT_THRESHOLD, message = 'Probing failed') {
        const isIncidentThresholdMet = incidentRetryAttempt === incidentThreshold - 1;
        if (!isIncidentThresholdMet) {
            this.logMessage(false, `${message}. Will try again. Attempt (${incidentRetryAttempt + 1}) with incident threshold (${incidentThreshold}).`);
            // throw here so that the retry function in src/components/probe/index.ts can retry again
            throw new Error(`${message} but incident threshold is not met.`);
        }
    }
    async sendNotification({ requestURL, notificationType, validation, alertId, }) {
        const isRecoveryNotification = notificationType === NotificationType.Recover;
        (0, events_2.getEventEmitter)().emit(events_1.default.probe.notification.willSend, {
            probeID: this.probeConfig.id,
            notifications: this.notifications,
            url: requestURL,
            probeState: isRecoveryNotification ? ProbeState.Up : ProbeState.Down,
            validation,
            alertId,
        });
        if (!this.hasNotification()) {
            return;
        }
        await (0, notification_1.sendAlerts)({
            probeID: this.probeConfig.id,
            url: requestURL,
            probeState: isRecoveryNotification ? ProbeState.Up : ProbeState.Down,
            notifications: this.notifications,
            validation,
        });
        await Promise.all(this.notifications.map((notification) => (0, history_1.saveNotificationLog)(this.probeConfig, notification, isRecoveryNotification
            ? NotificationType.Recover
            : NotificationType.Incident, '')));
    }
    handleFailedProbe(probeResults) {
        const hasfailedProbe = probeResults.find(({ requestResponse }) => requestResponse.result !== request_1.probeRequestResult.success);
        const { requestResponse } = hasfailedProbe;
        const requestIndex = probeResults.findIndex(({ requestResponse }) => requestResponse.result !== request_1.probeRequestResult.success);
        const failedRequestAssertion = this.getFailedRequestAssertion(requestIndex);
        if (!failedRequestAssertion) {
            pino_1.log.error('Failed request assertion is not found');
            return;
        }
        (0, events_2.getEventEmitter)().emit(events_1.default.probe.alert.triggered, {
            probe: this.probeConfig,
            requestIndex,
            alertQuery: failedRequestAssertion,
        });
        (0, incident_1.addIncident)({
            alert: failedRequestAssertion,
            probeID: this.probeConfig.id,
            probeRequestURL: this.probeConfig?.requests?.[requestIndex].url || '',
        });
        (0, history_1.saveProbeRequestLog)({
            probe: this.probeConfig,
            requestIndex,
            probeRes: requestResponse,
            alertQueries: [failedRequestAssertion.assertion],
            error: requestResponse.error,
        });
        const url = this.probeConfig?.requests?.[requestIndex].url || '';
        const validation = {
            alert: failedRequestAssertion,
            isAlertTriggered: true,
            response: requestResponse,
        };
        const probeID = this.probeConfig.id;
        const alertId = (0, alert_id_1.getAlertID)(url, validation, probeID);
        this.sendNotification({
            requestURL: url,
            notificationType: NotificationType.Incident,
            validation,
            alertId,
        }).catch((error) => pino_1.log.error(error.mesage));
    }
    handleRecovery(probeResults) {
        const recoveredIncident = (0, incident_1.getIncidents)().find((incident) => incident.probeID === this.probeConfig.id);
        const requestIndex = this.probeConfig?.requests?.findIndex(({ url }) => url === recoveredIncident?.probeRequestURL) || 0;
        if (recoveredIncident) {
            (0, incident_1.removeIncident)({
                probeID: this.probeConfig.id,
            });
            const url = this.probeConfig?.requests?.[requestIndex].url || '';
            const validation = {
                alert: recoveredIncident.alert,
                isAlertTriggered: false,
                response: probeResults[requestIndex].requestResponse,
            };
            const probeID = this.probeConfig.id;
            const alertId = (0, alert_id_1.getAlertID)(url, validation, probeID);
            this.sendNotification({
                requestURL: url,
                notificationType: NotificationType.Recover,
                validation,
                alertId,
            }).catch((error) => pino_1.log.error(error.mesage));
        }
        (0, history_1.saveProbeRequestLog)({
            probe: this.probeConfig,
            requestIndex,
            probeRes: probeResults[requestIndex].requestResponse,
            alertQueries: [recoveredIncident?.alert.assertion || ''],
        });
    }
    logMessage(isSuccess, ...message) {
        if (isSuccess) {
            pino_1.log.info(`${this.getMessagePrefix()} ${message.filter(Boolean).join(', ')}`);
            return;
        }
        pino_1.log.warn(`${this.getMessagePrefix()} ${message.filter(Boolean).join(', ')}`);
    }
    initializeProbeState() {
        if (!this.probeConfig?.lastEvent ||
            this.probeConfig.lastEvent?.recoveredAt !== null) {
            return;
        }
        if (!this.probeConfig.lastEvent?.alertId) {
            pino_1.log.error(`Last event ID in probe ${this.probeConfig.id} is required`);
            return;
        }
        const { alertId, createdAt } = this.probeConfig.lastEvent;
        const alert = this.getAlerts().find(({ id }) => id === alertId);
        if (!alert) {
            pino_1.log.error(`Alert ID: ${alertId} is not found in probe ${this.probeConfig.id}`);
            return;
        }
        const request = this.getRequestByAlertId(alertId);
        if (!request) {
            pino_1.log.error(`Request for alert ID: ${alertId} is not found in probe ${this.probeConfig.id}`);
            return;
        }
        (0, incident_1.addIncident)({
            alert,
            probeID: this.probeConfig.id,
            probeRequestURL: request.url,
            createdAt,
        });
    }
    getAlerts() {
        const httpAlerts = this.probeConfig?.requests?.map(({ alerts }) => alerts).find(Boolean) ||
            [];
        const mariadbAlerts = this.probeConfig?.mariadb?.map(({ alerts }) => alerts).find(Boolean) || [];
        const mongoAlerts = this.probeConfig?.mongo?.map(({ alerts }) => alerts).find(Boolean) || [];
        const mysqlAlerts = this.probeConfig?.mysql?.map(({ alerts }) => alerts).find(Boolean) || [];
        const pingAlerts = this.probeConfig?.ping?.map(({ alerts }) => alerts).find(Boolean) || [];
        const postgresAlerts = this.probeConfig?.postgres?.map(({ alerts }) => alerts).find(Boolean) ||
            [];
        const redisAlerts = this.probeConfig?.redis?.map(({ alerts }) => alerts).find(Boolean) || [];
        const socketAlerts = this.probeConfig?.socket?.alerts || [];
        return [
            ...(this.probeConfig?.alerts || []),
            ...httpAlerts,
            ...mariadbAlerts,
            ...mongoAlerts,
            ...mysqlAlerts,
            ...pingAlerts,
            ...postgresAlerts,
            ...redisAlerts,
            ...socketAlerts,
        ];
    }
    getRequestByAlertId(alertId) {
        if (!this.probeConfig.requests) {
            return null;
        }
        for (const request of this.probeConfig.requests) {
            if (!request?.alerts) {
                continue;
            }
            if (request.alerts.some(({ id }) => id === alertId)) {
                return request;
            }
        }
        return null;
    }
    hasNotification() {
        return this.notifications.length > 0;
    }
    getMessagePrefix() {
        return `${new Date().toISOString()} ${this.counter} id:${this.probeConfig.id}`;
    }
}
exports.BaseProber = BaseProber;
//# sourceMappingURL=index.js.map