"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mongoRequest = void 0;
const mongodb_1 = require("mongodb");
const mongodbURI = __importStar(require("mongodb-uri"));
const request_1 = require("../../../../interfaces/request");
const date_fns_1 = require("date-fns");
const catch_error_handler_1 = require("../../../../utils/catch-error-handler");
/**
 * MongoRequest is the interface to call Mongo and manage the request-response
 * @param {object} params is a MongoRequest type
 * @returns {object} ProbeRequestResponse type mapped from MongoResult
 **/
async function mongoRequest(params) {
    const baseResponse = {
        requestType: 'mongo',
        data: '',
        body: '',
        status: 0,
        result: request_1.probeRequestResult.unknown,
        headers: '',
        responseTime: 0,
    };
    const startTime = new Date();
    const result = await sendMongoRequest(params);
    const endTime = new Date();
    const duration = (0, date_fns_1.differenceInMilliseconds)(endTime, startTime);
    if (result.isAlive) {
        baseResponse.responseTime = duration;
        baseResponse.body = result.message;
        baseResponse.status = 200;
        baseResponse.result = request_1.probeRequestResult.success;
    }
    else {
        baseResponse.result = request_1.probeRequestResult.failed;
        baseResponse.body = '';
        baseResponse.error = result.message;
    }
    return baseResponse;
}
exports.mongoRequest = mongoRequest;
/**
 * sendMongoRequest actually sends the command/request to Mongo
 * @param {object} params is a MongoRequest type
 * @returns {object} MongoResult type contain client response
 */
async function sendMongoRequest(params) {
    const { uri } = params;
    let host;
    let port;
    let username;
    let password;
    let connectionURI;
    if (uri) {
        const parsed = mongodbURI.parse(uri);
        host = parsed.hosts[0].host;
        port = parsed.hosts[0].port;
        username = parsed.username;
        password = parsed.password;
        connectionURI = uri;
    }
    else {
        host = params.host;
        port = params.port;
        username = params.username;
        password = params.password;
        connectionURI = mongodbURI.format({
            scheme: 'mongodb',
            hosts: [
                {
                    host: host,
                    port,
                },
            ],
            username,
            password,
        });
    }
    const result = {
        isAlive: false,
        message: '',
    };
    const client = new mongodb_1.MongoClient(connectionURI, {
        connectTimeoutMS: 3000,
        maxIdleTimeMS: 3000,
        serverSelectionTimeoutMS: 3000,
    });
    try {
        await client.connect();
        client.on('error', (error) => {
            result.message = error;
        });
        const { ok } = await client.db().command({ ping: 1 });
        if (ok === 1) {
            result.isAlive = true;
            result.message = `${host}:${port} PONGED`;
        }
    }
    catch (error) {
        result.message = (0, catch_error_handler_1.getErrorMessage)(error);
    }
    finally {
        if (client) {
            await client.close();
        }
    }
    return result;
}
//# sourceMappingURL=request.js.map