"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.probePostgres = exports.PostgresProber = void 0;
const pg_connection_string_1 = require("pg-connection-string");
const __1 = require("..");
const request_1 = require("../../../../interfaces/request");
const request_2 = require("./request");
class PostgresProber extends __1.BaseProber {
    async probe({ incidentRetryAttempt }) {
        if (!this.probeConfig.postgres) {
            throw new Error(`Postgres configuration is empty. Probe ID: ${this.probeConfig.id}`);
        }
        const result = await probePostgres({
            id: this.probeConfig.id,
            checkOrder: this.counter,
            postgres: this.probeConfig.postgres,
        });
        this.processProbeResults(result, incidentRetryAttempt);
    }
    generateVerboseStartupMessage() {
        const { description, id, interval, name } = this.probeConfig;
        let result = `- Probe ID: ${id}
  Name: ${name || '-'}
  Description: ${description || '-'}
  Interval: ${interval}
`;
        result += '  Connection Details:';
        result += this.getConnectionDetails();
        return result;
    }
    getConnectionDetails() {
        return (this.probeConfig.postgres
            ?.map((db) => {
            if (db.uri) {
                return `
    URI: ${db.uri}
`;
            }
            return `
    Host: ${db.host}
    Port: ${db.port}
    Username: ${db.username}
    Database: ${db.database}
`;
        })
            .join('\n') || '');
    }
}
exports.PostgresProber = PostgresProber;
async function probePostgres({ id, checkOrder, postgres, }) {
    const probeResults = [];
    for await (const pg of postgres) {
        const postgresConnectionDetails = getPostgresConnectionDetails(pg);
        const { host, port } = postgresConnectionDetails;
        const requestResponse = await (0, request_2.postgresRequest)(postgresConnectionDetails);
        const { body, responseTime, result } = requestResponse;
        const isAlertTriggered = result !== request_1.probeRequestResult.success;
        const timeNow = new Date().toISOString();
        const logMessage = `${timeNow} ${checkOrder} id:${id} postgres:${host}:${port} ${responseTime}ms msg:${body}`;
        probeResults.push({ isAlertTriggered, logMessage, requestResponse });
    }
    return probeResults;
}
exports.probePostgres = probePostgres;
function getPostgresConnectionDetails(postgres) {
    const { uri } = postgres;
    if (!uri) {
        return postgres;
    }
    // If got uri format, parse and use that instead
    const { database, host, password, port, user } = (0, pg_connection_string_1.parse)(uri);
    return {
        host: host || '0.0.0.0',
        port: Number(port) || 5432,
        database: database || '',
        username: user || '',
        password: password || '',
    };
}
//# sourceMappingURL=index.js.map