"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
exports.postgresRequest = void 0;
const pg_1 = require("pg");
const request_1 = require("../../../../interfaces/request");
const date_fns_1 = require("date-fns");
const catch_error_handler_1 = require("../../../../utils/catch-error-handler");
async function postgresRequest(params) {
    const baseResponse = {
        requestType: 'postgres',
        data: '',
        body: '',
        status: 0,
        headers: '',
        responseTime: 0,
        result: request_1.probeRequestResult.unknown,
    };
    const startTime = new Date();
    const result = await sendPsqlRequest(params);
    const endTime = new Date();
    const duration = (0, date_fns_1.differenceInMilliseconds)(endTime, startTime);
    if (result.isAlive) {
        baseResponse.responseTime = duration;
        baseResponse.body = result.message;
        baseResponse.status = 200;
        baseResponse.result = request_1.probeRequestResult.success;
    }
    else {
        baseResponse.result = request_1.probeRequestResult.failed;
        baseResponse.body = '';
        baseResponse.error = result.message;
    }
    return baseResponse;
}
exports.postgresRequest = postgresRequest;
const IDLETIMEOUTMS = 10000; // Default idle timeout
const CONNECTIONTIMEOUTMS = 10000; // Default connection timout
async function sendPsqlRequest(params) {
    const result = {
        isAlive: false,
        message: '',
    };
    let client;
    try {
        const pool = new pg_1.Pool({
            host: params.host,
            port: params.port,
            database: params.database,
            user: params.username || '',
            password: params.password || '',
            idleTimeoutMillis: IDLETIMEOUTMS,
            connectionTimeoutMillis: CONNECTIONTIMEOUTMS,
        });
        client = await pool.connect();
        await client.query('SELECT NOW()');
        result.message = 'postgres ok';
        result.isAlive = true;
    }
    catch (error) {
        result.message = (0, catch_error_handler_1.getErrorMessage)(error);
    }
    finally {
        if (client) {
            // release if connect was previously successful.
            client.release(true);
        }
    }
    return result;
}
//# sourceMappingURL=request.js.map