"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.probeRedis = exports.RedisProber = void 0;
const url_parse_1 = __importDefault(require("url-parse"));
const __1 = require("..");
const request_1 = require("../../../../interfaces/request");
const request_2 = require("./request");
class RedisProber extends __1.BaseProber {
    async probe({ incidentRetryAttempt }) {
        if (!this.probeConfig.redis) {
            throw new Error(`Redis configuration is empty. Probe ID: ${this.probeConfig.id}`);
        }
        const result = await probeRedis({
            id: this.probeConfig.id,
            checkOrder: this.counter,
            redis: this.probeConfig.redis,
        });
        this.processProbeResults(result, incidentRetryAttempt);
    }
    generateVerboseStartupMessage() {
        const { description, id, interval, name } = this.probeConfig;
        let result = `- Probe ID: ${id}
  Name: ${name}
  Description: ${description || '-'}
  Interval: ${interval}
`;
        result += '  Connection Details:';
        result += this.getConnectionDetails();
        return result;
    }
    getConnectionDetails() {
        return (this.probeConfig.redis
            ?.map((db) => {
            if (db.uri) {
                return `
    URI: ${db.uri}
`;
            }
            return `
    Host: ${db.host}
    Port: ${db.port}
    Username: ${db.username}
        `;
        })
            .join('\n') || '');
    }
}
exports.RedisProber = RedisProber;
async function probeRedis({ id, checkOrder, redis, }) {
    const probeResults = [];
    let hostTxt = '0.0.0.0';
    let portTxt = 6379;
    for await (const { host, port, uri, username, password } of redis) {
        const requestResponse = await (0, request_2.redisRequest)({
            host,
            port,
            username,
            password,
            uri,
        });
        const { body, responseTime, result } = requestResponse;
        const isAlertTriggered = result !== request_1.probeRequestResult.success;
        const timeNow = new Date().toISOString();
        // parse uri for logMessage(host:port)
        if (host && port) {
            hostTxt = host;
            portTxt = port;
        }
        else if (uri) {
            const parsed = (0, url_parse_1.default)(uri);
            hostTxt = parsed.host;
            portTxt = Number(parsed.port);
        }
        const logMessage = `${timeNow} ${checkOrder} id:${id} redis:${hostTxt}:${portTxt} ${responseTime}ms msg:${body}`;
        probeResults.push({ isAlertTriggered, logMessage, requestResponse });
    }
    return probeResults;
}
exports.probeRedis = probeRedis;
//# sourceMappingURL=index.js.map