"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
exports.redisRequest = void 0;
const redis_1 = require("redis");
const date_fns_1 = require("date-fns");
const request_1 = require("../../../../interfaces/request");
const catch_error_handler_1 = require("../../../../utils/catch-error-handler");
/**
 * redisRequest is the interface to call redis and manage the request-response
 * @param {object} params is a RedisRequest type
 * @returns {object} ProbeRequestResponse type mapped from RedisResult
 **/
async function redisRequest(params) {
    const baseResponse = {
        requestType: 'redis',
        data: '',
        body: '',
        status: 0,
        headers: '',
        responseTime: 0,
        result: request_1.probeRequestResult.unknown,
    };
    const startTime = new Date();
    const result = await sendRedisRequest(params);
    const endTime = new Date();
    const duration = (0, date_fns_1.differenceInMilliseconds)(endTime, startTime);
    if (result.isAlive) {
        baseResponse.responseTime = duration;
        baseResponse.body = result.message;
        baseResponse.status = 200;
        baseResponse.result = request_1.probeRequestResult.success;
    }
    else {
        baseResponse.body = '';
        baseResponse.result = request_1.probeRequestResult.failed;
        baseResponse.error = result.message;
    }
    return baseResponse;
}
exports.redisRequest = redisRequest;
const CONNECTTIMEOUTMS = 10000;
/**
 * sendRedisRequest actually sends the command/request to redis
 * @param {object} params is a RedisRequest type
 * @returns {object} RedisResult type contain client response
 */
async function sendRedisRequest(params) {
    const { host, port, username, password, uri } = params;
    const result = {
        isAlive: false,
        message: '',
    };
    try {
        const client = host && port
            ? (0, redis_1.createClient)({
                socket: {
                    host,
                    port,
                    connectTimeout: CONNECTTIMEOUTMS,
                },
                password,
                username,
            })
            : (0, redis_1.createClient)({ url: uri });
        await client.connect();
        client.on('error', (error) => {
            result.message = error;
        });
        const ping = await client.ping();
        if (ping === 'PONG') {
            result.isAlive = true;
            result.message = `redis PONGED`;
        }
    }
    catch (error) {
        result.message = (0, catch_error_handler_1.getErrorMessage)(error);
    }
    return result;
}
//# sourceMappingURL=request.js.map