"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tcpRequest = void 0;
const net_1 = __importDefault(require("net"));
const date_fns_1 = require("date-fns");
const request_1 = require("../../../../interfaces/request");
const catch_error_handler_1 = require("../../../../utils/catch-error-handler");
// tcpRequest sends out the tcp request and returns a monika standard request response
async function tcpRequest(param) {
    const { host, port, data, timeout } = param;
    const tcpResp = await tcpCheck({ host, port, data, timeout });
    // map tcpResp to standard monika baseResponse
    const baseResponse = {
        requestType: 'tcp',
        data: '',
        body: tcpResp.responseData || tcpResp.message || '-',
        status: tcpResp.status === 'DOWN' ? 0 : 200,
        result: tcpResp.status === 'DOWN'
            ? request_1.probeRequestResult.failed
            : request_1.probeRequestResult.success,
        headers: {},
        responseTime: tcpResp.duration,
        error: tcpResp.message, // map message if any
    };
    return baseResponse;
}
exports.tcpRequest = tcpRequest;
async function tcpCheck(tcpRequest) {
    const { host, port, data, timeout } = tcpRequest;
    try {
        const startTime = new Date();
        const resp = await sendTCP({ host, port, data, timeout });
        const endTime = new Date();
        const duration = (0, date_fns_1.differenceInMilliseconds)(endTime, startTime);
        return {
            duration,
            responseData: resp,
            status: 'UP',
            message: '',
        };
    }
    catch (error) {
        return {
            duration: 0,
            responseData: null,
            status: 'DOWN',
            message: (0, catch_error_handler_1.getErrorMessage)(error),
        };
    }
}
const SOCKETTIMEOUTMS = 10000;
async function sendTCP(tcpRequest) {
    const { host, port, data, timeout } = tcpRequest;
    return new Promise((resolve, reject) => {
        const client = net_1.default.createConnection({ host, port }, () => {
            data ? client.write(data) : client.end();
        });
        client.setTimeout(timeout || SOCKETTIMEOUTMS);
        client.on('data', (data) => {
            resolve(data);
            client.end();
        });
        client.on('close', (hadError) => {
            if (hadError)
                reject(new Error('unknown'));
            resolve(null);
        });
        client.on('timeout', () => {
            reject(new Error('timeout'));
            client.end();
        });
        client.on('error', (error) => {
            reject(error);
            client.end();
        });
    });
}
//# sourceMappingURL=request.js.map