"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.savePidFile = exports.getSummaryAndSendNotif = void 0;
const node_fs_1 = __importDefault(require("node:fs"));
const node_os_1 = require("node:os");
const monika_notification_1 = require("@hyperjumptech/monika-notification");
const format_1 = __importDefault(require("date-fns/format"));
const config_1 = require("../components/config");
const history_1 = require("../components/logger/history");
const response_time_log_1 = require("../components/logger/response-time-log");
const alert_message_1 = require("../components/notification/alert-message");
const context_1 = require("../context");
const events_1 = __importDefault(require("../events"));
const events_2 = require("../utils/events");
const catch_error_handler_1 = require("../utils/catch-error-handler");
const ip_1 = __importDefault(require("../utils/ip"));
const pino_1 = require("../utils/pino");
const public_ip_1 = require("../utils/public-ip");
const eventEmitter = (0, events_2.getEventEmitter)();
async function getSummaryAndSendNotif() {
    const config = (0, config_1.getValidatedConfig)();
    const { notifications, probes } = config;
    if (!notifications)
        return;
    try {
        const { userAgent } = (0, context_1.getContext)();
        const privateIpAddress = (0, ip_1.default)();
        const [summary, osName, monikaInstance] = await Promise.all([
            (0, history_1.getSummary)(),
            (0, alert_message_1.getOSName)(),
            (0, alert_message_1.getMonikaInstance)(privateIpAddress),
        ]);
        const { numberOfIncidents, numberOfRecoveries, numberOfSentNotifications } = summary;
        const responseTimelogLifeTimeInHour = (0, response_time_log_1.getLogLifeTimeInHour)();
        const numberOfProbes = probes.length;
        const tweetMessage = createTweetMessage({
            averageResponseTime: response_time_log_1.averageResponseTime,
            numberOfIncidents,
            numberOfProbes,
            numberOfRecoveries,
        });
        (0, monika_notification_1.sendNotifications)(notifications, {
            subject: `Monika Status`,
            body: `Status Update ${(0, format_1.default)(new Date(), 'yyyy-MM-dd HH:mm:ss XXX')}
Host: ${monikaInstance})
Number of probes: ${numberOfProbes}
Maximum response time: ${response_time_log_1.maxResponseTime} ms in the last ${responseTimelogLifeTimeInHour} hours
Minimum response time: ${response_time_log_1.minResponseTime} ms in the last ${responseTimelogLifeTimeInHour} hours
Average response time: ${response_time_log_1.averageResponseTime} ms in the last ${responseTimelogLifeTimeInHour} hours
Incidents: ${numberOfIncidents} in the last 24 hours
Recoveries: ${numberOfRecoveries} in the last 24 hours
Notifications: ${numberOfSentNotifications}
OS: ${osName}
Version: ${userAgent}

${tweetMessage}
`,
            summary: `There are ${numberOfIncidents} incidents and ${numberOfRecoveries} recoveries in the last 24 hours. - ${userAgent} - ${osName}`,
            meta: {
                type: 'status-update',
                time: (0, format_1.default)(new Date(), 'yyyy-MM-dd HH:mm:ss XXX'),
                hostname: (0, node_os_1.hostname)(),
                privateIpAddress,
                publicIpAddress: public_ip_1.publicIpAddress,
                monikaInstance,
                maxResponseTime: response_time_log_1.maxResponseTime,
                minResponseTime: response_time_log_1.minResponseTime,
                averageResponseTime: response_time_log_1.averageResponseTime,
                responseTimelogLifeTimeInHour,
                version: userAgent,
                numberOfProbes,
                ...summary,
            },
        }).catch((error) => pino_1.log.error(`Summary notification: ${error.message}`));
        if ((0, response_time_log_1.checkIs24HourHasPassed)()) {
            (0, response_time_log_1.resetlogs)();
        }
    }
    catch (error) {
        pino_1.log.error(`Summary notification: ${(0, catch_error_handler_1.getErrorMessage)(error)}`);
    }
}
exports.getSummaryAndSendNotif = getSummaryAndSendNotif;
// savePidFile saves a monika.pid file with some useful information
function savePidFile(configFile, { notifications, probes }) {
    const data = JSON.stringify({
        monikaStartTime: new Date(),
        monikaConfigFile: configFile,
        monikaPid: process.pid,
        monikaProbes: probes.length,
        monikaNotifs: notifications.length,
    });
    node_fs_1.default.writeFile('monika.pid', data, (err) => {
        if (err) {
            pino_1.log.info("couldn't save monika.pid file, got err: ", err);
        }
    });
}
exports.savePidFile = savePidFile;
// do somee cleanups on exit
eventEmitter.on(events_1.default.application.terminated, async () => {
    node_fs_1.default.unlink('monika.pid', (err) => {
        if (err) {
            pino_1.log.info('trying to cleanup monika.pid, got err: ', err);
        }
    });
});
function createTweetMessage({ averageResponseTime, numberOfIncidents, numberOfProbes, numberOfRecoveries, }) {
    const message = `I am using Monika by @hyperjump_tech to monitor ${numberOfProbes} probes! In the last 24 hours,

⏱ the average response time is ${averageResponseTime} ms
⚠️ there were ${numberOfIncidents} incidents
✅ and ${numberOfRecoveries} recoveries!

Give it a try!

https://monika.hyperjump.tech`;
    return `<a href=https://twitter.com/intent/tweet?text=${encodeURI(message)}&hashtags=opensource,monika>Tweet this status!</a>`;
}
//# sourceMappingURL=summary-notification.js.map