"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tlsChecker = void 0;
const monika_notification_1 = require("@hyperjumptech/monika-notification");
const date_fns_1 = require("date-fns");
const config_1 = require("../components/config");
const history_1 = require("../components/logger/history");
const alert_message_1 = require("../components/notification/alert-message");
const tls_checker_1 = require("../components/tls-checker");
const context_1 = require("../context");
const catch_error_handler_1 = require("../utils/catch-error-handler");
const ip_1 = __importDefault(require("../utils/ip"));
const pino_1 = require("../utils/pino");
const public_ip_1 = require("../utils/public-ip");
function tlsChecker() {
    const { certificate, notifications } = (0, config_1.getValidatedConfig)();
    if (!certificate) {
        return;
    }
    const hasDomain = certificate.domains.length > 0;
    if (!hasDomain) {
        return;
    }
    const { domains, reminder } = certificate;
    const hasNotification = notifications.length > 0;
    for (const domain of domains) {
        const hostname = (0, tls_checker_1.getHostname)(domain);
        pino_1.log.info(`Running TLS check for ${hostname} every day at 00:00`);
        (0, tls_checker_1.checkTLS)(domain, reminder).catch((error) => {
            const errorMessage = (0, catch_error_handler_1.getErrorMessage)(error);
            pino_1.log.error(errorMessage);
            if (!hasNotification) {
                return;
            }
            sendErrorNotification({
                errorMessage,
                hostname,
                notifications,
            }).catch((error) => pino_1.log.error((0, catch_error_handler_1.getErrorMessage)(error)));
        });
    }
}
exports.tlsChecker = tlsChecker;
async function sendErrorNotification({ errorMessage, hostname, notifications, }) {
    const probe = {
        id: '',
        alerts: [],
        interval: 10,
        name: '',
        requests: [],
    };
    await Promise.allSettled(notifications.map(async (notification) => {
        await (0, monika_notification_1.sendNotifications)(notifications, await getNotificationMessage({ hostname, errorMessage }));
        await (0, history_1.saveNotificationLog)(probe, notification, 'NOTIFY-TLS', errorMessage);
    }));
}
async function getNotificationMessage({ hostname, errorMessage, }) {
    const privateIpAddress = (0, ip_1.default)();
    const { userAgent } = (0, context_1.getContext)();
    const [monikaInstance, osName] = await Promise.all([
        (0, alert_message_1.getMonikaInstance)(privateIpAddress),
        (0, alert_message_1.getOSName)(),
    ]);
    const time = (0, date_fns_1.format)(new Date(), 'yyyy-MM-dd HH:mm:ss XXX');
    const body = `Message: ${errorMessage}

  URL: ${hostname}

  Time: ${time}

  From: ${monikaInstance}

  OS: ${osName}

  Version: ${userAgent}`;
    return {
        subject: 'New Incident from Monika',
        body,
        summary: errorMessage,
        meta: {
            hostname,
            privateIpAddress,
            probeID: '',
            publicIpAddress: public_ip_1.publicIpAddress,
            time,
            type: 'incident',
            url: hostname,
            version: userAgent,
        },
    };
}
//# sourceMappingURL=tls-check.js.map