"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
exports.startProbing = exports.loopCheckSTUNServer = exports.FAILED_REQUEST_ASSERTION = exports.sanitizeProbe = void 0;
const uuid_1 = require("uuid");
const probe_1 = require("../components/config/probe");
const probe_2 = require("../components/probe");
const context_1 = require("../context");
const pino_1 = require("../utils/pino");
const probe_state_1 = require("../utils/probe-state");
const public_ip_1 = require("../utils/public-ip");
const default_values_1 = require("../components/config/validation/validator/default-values");
let checkSTUNinterval;
const DISABLE_STUN = -1; // -1 is disable stun checking
function sanitizeProbe(isSymonMode, probe) {
    const { id, name, requests, incidentThreshold, recoveryThreshold, alerts } = probe;
    if (!name) {
        pino_1.log.warn(`Warning: Probe ${id} has no name defined. Using the default name started by Monika`);
    }
    const isHTTPProbe = Boolean(requests);
    const isAlertsEmpty = alerts === undefined || alerts.length === 0;
    if (!isSymonMode && isHTTPProbe && isAlertsEmpty) {
        pino_1.log.warn(`Warning: Probe ${id} has no Alerts configuration defined. Using the default response.status != 200 and response.time > 2000`);
    }
    return {
        ...probe,
        incidentThreshold: incidentThreshold || default_values_1.DEFAULT_INCIDENT_THRESHOLD,
        recoveryThreshold: recoveryThreshold || default_values_1.DEFAULT_RECOVERY_THRESHOLD,
        alerts: isSymonMode ? [] : addFailedRequestAssertions(alerts),
    };
}
exports.sanitizeProbe = sanitizeProbe;
exports.FAILED_REQUEST_ASSERTION = {
    assertion: '',
    message: 'Probe not accessible',
};
function addFailedRequestAssertions(assertions) {
    // create uuid seed from FAILED_REQUEST_ASSERTION
    // to make the id deterministic, consistent, and testable
    const idSeed = Buffer.from(JSON.stringify(exports.FAILED_REQUEST_ASSERTION));
    return [
        ...assertions,
        {
            id: (0, uuid_1.v4)({
                random: idSeed,
            }),
            ...exports.FAILED_REQUEST_ASSERTION,
        },
    ];
}
async function loopCheckSTUNServer(interval) {
    // if stun is disabled, no need to create interval
    if (interval === -1)
        return;
    // if interval = 0 get ip once and exit. No need to setup interval.
    if (interval === 0 || (0, context_1.getContext)().isTest) {
        await (0, public_ip_1.getPublicIp)();
        return;
    }
    checkSTUNinterval = setInterval(async () => {
        await (0, public_ip_1.getPublicIp)();
    }, interval * 1000);
    return checkSTUNinterval;
}
exports.loopCheckSTUNServer = loopCheckSTUNServer;
function startProbing({ notifications, probes, signal, }) {
    (0, probe_state_1.initializeProbeStates)(probes);
    const probeInterval = setInterval(() => {
        if (signal.aborted) {
            clearInterval(probeInterval);
            return;
        }
        if (isEndOfRepeat((0, probe_1.getProbes)())) {
            // eslint-disable-next-line unicorn/no-process-exit, n/no-process-exit
            process.exit(0);
        }
        if (!isStunOK()) {
            return;
        }
        for (const probe of (0, probe_1.getProbes)()) {
            (0, probe_2.doProbe)({
                notifications,
                probe,
                signal,
            });
        }
    }, 1000);
}
exports.startProbing = startProbing;
function isEndOfRepeat(probes) {
    const isAllProbeFinished = probes.every(({ id }) => isLastCycleOf(id) && (0, probe_state_1.getProbeState)(id) !== 'running');
    return (0, context_1.getContext)().flags.repeat && isAllProbeFinished;
}
function isStunOK() {
    return (0, context_1.getContext)().flags.stun === DISABLE_STUN || public_ip_1.isConnectedToSTUNServer;
}
function isLastCycleOf(probeID) {
    const probeCtx = (0, probe_state_1.getProbeContext)(probeID);
    if (!probeCtx) {
        return true;
    }
    return (0, context_1.getContext)().flags.repeat === probeCtx.cycle;
}
//# sourceMappingURL=index.js.map