"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrometheusCollector = void 0;
const prom_client_1 = require("prom-client");
const request_1 = require("../../../interfaces/request");
let prometheusCustomCollector;
class PrometheusCollector {
    constructor() {
        // remove all registered metrics
        prom_client_1.register.clear();
        // register metric collector
        const alertsTriggered = new prom_client_1.Counter({
            name: 'monika_alerts_triggered',
            help: 'Indicates the count of alerts triggered',
            labelNames: ['id', 'name', 'url', 'method', 'alertQuery'],
        });
        const alertsTriggeredTotal = new prom_client_1.Counter({
            name: 'monika_alerts_triggered_total',
            help: 'Indicates the cumulative count of alerts triggered',
        });
        const probesRunning = new prom_client_1.Gauge({
            name: 'monika_probes_running',
            help: 'Indicates whether a a probe is running (1) or idle (0)',
            labelNames: ['id'],
        });
        const probesRunningTotal = new prom_client_1.Gauge({
            name: 'monika_probes_running_total',
            help: 'Indicates the total count of probes that are currently running checks',
        });
        const probesStatus = new prom_client_1.Gauge({
            name: 'monika_probes_status',
            help: 'Indicates whether a probe is healthy (1) or is having an incident (0)',
            labelNames: ['id', 'name', 'url', 'method'],
        });
        const probesTotal = new prom_client_1.Gauge({
            name: 'monika_probes_total',
            help: 'Total count of all probes configured',
        });
        const responseSize = new prom_client_1.Gauge({
            name: 'monika_request_response_size_bytes',
            help: "Indicates the size of probe request's response size in bytes",
            labelNames: [
                'id',
                'name',
                'url',
                'method',
                'statusCode',
                'result',
            ],
        });
        const responseTime = new prom_client_1.Histogram({
            name: 'monika_request_response_time_seconds',
            help: 'Indicates the duration of the probe request in seconds',
            labelNames: [
                'id',
                'name',
                'url',
                'method',
                'statusCode',
                'result',
            ],
        });
        const statusCode = new prom_client_1.Gauge({
            name: 'monika_request_status_code_info',
            help: "Indicates the HTTP status code of the probe requests' response(s)",
            labelNames: ['id', 'name', 'url', 'method'],
        });
        const notificationsTriggered = new prom_client_1.Counter({
            name: 'monika_notifications_triggered',
            help: 'Indicates the count of notifications triggered',
            labelNames: ['type', 'status'],
        });
        const notificationsTriggeredTotal = new prom_client_1.Counter({
            name: 'monika_notifications_triggered_total',
            help: 'Indicates the cumulative count of notifications triggered',
        });
        // register and collect default Node.js metrics
        (0, prom_client_1.collectDefaultMetrics)({ register: prom_client_1.register });
        prometheusCustomCollector = {
            alertsTriggered,
            alertsTriggeredTotal,
            probesRunningTotal,
            probesRunning,
            probesStatus,
            probesTotal,
            responseSize,
            responseTime,
            statusCode,
            notificationsTriggered,
            notificationsTriggeredTotal,
        };
    }
    collectProbeRequestMetrics(probeResult) {
        if (!prometheusCustomCollector) {
            throw new Error('Prometheus collector is not registered');
        }
        const { probe, requestIndex, response } = probeResult;
        const { id, name, requests } = probe;
        if (!requests || requests.length === 0) {
            return;
        }
        const request = requests[requestIndex];
        const { method, url } = request;
        const { headers, responseTime, status } = response;
        const result = response.result ?? request_1.probeRequestResult.unknown;
        const milliSecond = 1000;
        const responseTimeInSecond = responseTime / milliSecond || 0;
        const responseSizeBytes = typeof headers === 'string'
            ? undefined
            : Number(headers['content-length']);
        const labels = {
            id,
            name,
            url,
            result,
            method: method ?? 'GET',
            statusCode: status,
        };
        const { statusCode, responseTime: resposeTimeCollector, responseSize, } = prometheusCustomCollector;
        // collect metrics
        statusCode
            ?.labels({
            id,
            name,
            url,
            method: method ?? 'GET',
        })
            .set(status);
        resposeTimeCollector?.labels(labels).observe(responseTimeInSecond);
        responseSize?.labels(labels).set(responseSizeBytes || 0);
    }
    collectProbeStatus(probeResult) {
        if (!prometheusCustomCollector) {
            throw new Error('Prometheus collector is not registered');
        }
        const { probe, requestIndex, status } = probeResult;
        const { id, name, requests } = probe;
        if (!requests || requests.length === 0) {
            return;
        }
        const request = requests[requestIndex];
        const { method, url } = request;
        const labels = {
            id,
            name,
            url,
            method: method ?? 'GET',
        };
        const { probesStatus } = prometheusCustomCollector;
        // collect metrics
        probesStatus?.labels(labels).set(status === 'up' ? 1 : 0);
    }
    collectProbeTotal(total) {
        if (!prometheusCustomCollector) {
            throw new Error('Prometheus collector is not registered');
        }
        prometheusCustomCollector.probesTotal.set(total);
    }
    collectTriggeredAlert(probeResult) {
        if (!prometheusCustomCollector) {
            throw new Error('Prometheus collector is not registered');
        }
        const { alertQuery, probe, requestIndex } = probeResult;
        const { id, name, requests } = probe;
        if (!requests || requests.length === 0) {
            return;
        }
        const request = requests[requestIndex];
        const { method, url } = request;
        const labels = {
            id,
            name,
            url,
            method: method ?? 'GET',
            alertQuery: JSON.stringify(alertQuery),
        };
        const { alertsTriggered, alertsTriggeredTotal } = prometheusCustomCollector;
        // collect metrics
        alertsTriggered?.labels(labels).inc();
        alertsTriggeredTotal?.inc();
    }
    decrementProbeRunningTotal(id) {
        if (!prometheusCustomCollector) {
            throw new Error('Prometheus collector is not registered');
        }
        prometheusCustomCollector.probesRunning.labels(id).dec();
        prometheusCustomCollector.probesRunningTotal.dec();
    }
    incrementProbeRunningTotal(id) {
        if (!prometheusCustomCollector) {
            throw new Error('Prometheus collector is not registered');
        }
        prometheusCustomCollector.probesRunning.labels(id).inc();
        prometheusCustomCollector.probesRunningTotal.inc();
    }
    resetProbeRunningTotal() {
        if (!prometheusCustomCollector) {
            throw new Error('Prometheus collector is not registered');
        }
        prometheusCustomCollector.probesRunning.reset();
        prometheusCustomCollector.probesRunningTotal.reset();
    }
    collectNotificationSentMetrics({ type, status, }) {
        if (!prometheusCustomCollector) {
            throw new Error('Prometheus collector is not registered');
        }
        const { notificationsTriggered, notificationsTriggeredTotal } = prometheusCustomCollector;
        // collect metrics
        notificationsTriggered.labels({ type, status }).inc();
        notificationsTriggeredTotal.inc();
    }
}
exports.PrometheusCollector = PrometheusCollector;
//# sourceMappingURL=collector.js.map