"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableAutoUpdate = void 0;
const fs_extra_1 = require("fs-extra");
const os = __importStar(require("os"));
const timers_1 = require("timers");
const pino_1 = require("../../utils/pino");
const date_fns_1 = require("date-fns");
const child_process_1 = require("child_process");
const unzipper = __importStar(require("unzipper"));
const hasha_1 = __importDefault(require("hasha"));
const http_1 = require("../../utils/http");
const DEFAULT_UPDATE_CHECK = 86400; // 24 hours
async function enableAutoUpdate(config, mode) {
    const installation = installationType(process.argv);
    if (process.env.NODE_ENV !== 'development' &&
        installation === 'binary' &&
        config.arch !== 'x64') {
        throw new TypeError('Updater: Monika binary only supports x64 architecture.');
    }
    mode = mode.toLowerCase();
    if (mode === 'major' || mode === 'minor' || mode === 'patch') {
        const updateMode = mode;
        await runUpdater(config, updateMode).catch((error) => pino_1.log.error(error));
        (0, timers_1.setInterval)(() => {
            runUpdater(config, updateMode).catch((error) => pino_1.log.error(error));
        }, DEFAULT_UPDATE_CHECK * 1000);
        return;
    }
    throw new TypeError(`Invalid auto-update ${mode}`);
}
exports.enableAutoUpdate = enableAutoUpdate;
/**
 * runUpdater compares current running version to recent released version from npm
 * @param config oclif config
 * @param updateMode major, minor, patch
 * @returns string of remote version to update
 * @returns undefined if no need to update
 */
async function runUpdater(config, updateMode) {
    pino_1.log.info('Updater: starting');
    const currentVersion = config.version;
    const { data } = await (0, http_1.sendHttpRequest)({
        url: 'https://registry.npmjs.org/@hyperjumptech/monika',
    });
    const latestVersion = data['dist-tags'].latest;
    if (latestVersion === currentVersion || config.debug) {
        const nextCheck = new Date(Date.now() + DEFAULT_UPDATE_CHECK * 1000);
        const date = (0, date_fns_1.format)(nextCheck, 'yyyy-MM-dd HH:mm:ss XXX');
        pino_1.log.info(`Updater: already running latest version, next check at ${date}.`);
        return;
    }
    if (updateMode === 'major') {
        await updateMonika(config, latestVersion);
        return;
    }
    const [currentMajor, currentMinor] = currentVersion.split('.');
    const { time } = data;
    // versions: key-value data with semver as key and timestamp as value
    // sorted descending by timestamp
    const versions = Object.keys(time);
    const predicate = (remoteVersion) => {
        let regexp = '';
        switch (updateMode) {
            case 'patch': {
                regexp = `(${currentMajor}\\.${currentMinor}\\.([0-9])+)`;
                break;
            }
            case 'minor': {
                regexp = `(${currentMajor}\\.([0-9])+\\.([0-9])+)`;
                break;
            }
            default: {
                return false;
            }
        }
        return remoteVersion.match(regexp) !== null;
    };
    const compatibleVersion = versions.reverse().find((v) => predicate(v));
    if (compatibleVersion === currentVersion) {
        const nextCheck = new Date(Date.now() + DEFAULT_UPDATE_CHECK * 1000);
        const date = (0, date_fns_1.format)(nextCheck, 'yyyy-MM-dd HH:mm:ss XXX');
        pino_1.log.info(`Updater: already running latest ${updateMode.toLowerCase()} version: ${currentVersion}. Next check at ${date}.`);
        return;
    }
    if (compatibleVersion !== undefined && compatibleVersion !== currentVersion) {
        await updateMonika(config, compatibleVersion);
    }
}
async function updateMonika(config, remoteVersion) {
    pino_1.log.info(`Updater: updating monika to v${remoteVersion}`);
    const installation = installationType(process.argv);
    if (installation === 'npm') {
        try {
            await spawnAsync(`npm install -g @hyperjumptech/monika@${remoteVersion}`, true);
        }
        catch (error) {
            pino_1.log.error(`Updater: npm install error, ${error}`);
        }
        pino_1.log.warn(`Monika has been updated to v${remoteVersion}, quitting...`);
        process.kill(process.pid, 'SIGINT');
        return;
    }
    // not npm installation, extract tarball release to monika installation path
    const downloadPath = await downloadMonika(config, remoteVersion);
    const extractPath = `${os.tmpdir()}/monika-${remoteVersion}`;
    await (0, fs_extra_1.remove)(extractPath);
    await (0, fs_extra_1.mkdirs)(extractPath);
    await extractArchive(downloadPath, extractPath);
    await moveExtractedFiles(config, extractPath);
    await (0, fs_extra_1.remove)(downloadPath);
    pino_1.log.warn(`Monika has been updated to v${remoteVersion}, quitting...`);
    process.kill(process.pid, 'SIGINT');
}
async function spawnAsync(command, detached) {
    return new Promise((resolve) => {
        const commands = command.split(' ');
        const cmd = (0, child_process_1.spawn)(commands[0], commands.slice(1), {
            detached,
            cwd: os.homedir(),
        });
        cmd.on('close', () => {
            resolve();
        });
    });
}
async function extractArchive(source, destination) {
    const archiveEntries = (0, fs_extra_1.createReadStream)(source).pipe(
    // eslint-disable-next-line new-cap
    unzipper.Parse({ forceStream: true }));
    const extractPromises = [];
    for await (const entry of archiveEntries) {
        const promise = new Promise((resolve, reject) => {
            const unzipperEntry = entry;
            const filename = unzipperEntry.path;
            const filepath = `${destination}/${filename}`;
            unzipperEntry
                .pipe((0, fs_extra_1.createWriteStream)(filepath))
                .on('error', (err) => reject(err))
                .on('close', () => resolve(filepath));
        });
        extractPromises.push(promise);
    }
    await Promise.all(extractPromises);
}
/**
 * moves files under @param source directory to monika installation path
 * @param config oclif config
 * @param source source file path to move
 * @returns void
 */
async function moveExtractedFiles(config, source) {
    const commandPath = process.argv[0];
    const commandRealPath = await (0, fs_extra_1.realpath)(commandPath);
    const commandsDirs = commandRealPath.split('/');
    let installationDirectory = commandsDirs.slice(0, -1).join('/');
    if (installationType(process.argv) === 'oclif-pack') {
        installationDirectory = commandsDirs.slice(0, -2).join('/');
    }
    const files = await (0, fs_extra_1.readdir)(source).then((filenames) => filenames.map((filename) => `${source}/${filename}`));
    for await (const filePath of files) {
        const source = filePath;
        const filename = source.split('/').splice(-1)[0];
        const destFile = `${installationDirectory}/${filename}`;
        try {
            await (0, fs_extra_1.move)(source, destFile, { overwrite: true });
            if (getPlatform(config) !== 'windows' &&
                destFile.match('/monika$') !== null) {
                await (0, fs_extra_1.chmod)(destFile, 0o755);
            }
        }
        catch (error) {
            pino_1.log.error(`Updater: move files error ${error}`);
        }
    }
}
/**
 * downloadMonika downloads binary releases from github
 *
 * @param config oclif config instance
 * @param remoteVersion version to download
 * @returns downloaded file path
 */
async function downloadMonika(config, remoteVersion) {
    const platformName = getPlatform(config);
    const filename = `monika-v${remoteVersion}-${platformName}-x64`;
    const downloadUri = `https://github.com/hyperjumptech/monika/releases/download/v${remoteVersion}/${filename}.zip`;
    pino_1.log.info(`Updater: download from ${downloadUri}`);
    const { data: downloadStream } = await (0, http_1.sendHttpRequest)({
        url: downloadUri,
        responseType: 'stream',
    });
    const { data: checksum } = await (0, http_1.sendHttpRequest)({
        url: `https://github.com/hyperjumptech/monika/releases/download/v${remoteVersion}/${filename}-CHECKSUM.txt`,
    });
    return new Promise((resolve, reject) => {
        const targetPath = `${os.tmpdir()}/${filename}`;
        const writer = (0, fs_extra_1.createWriteStream)(targetPath);
        const stream = downloadStream;
        stream.pipe(writer);
        writer.on('error', (err) => {
            writer.close();
            reject(err);
        });
        writer.on('close', () => {
            pino_1.log.info(`Updater: verifying download`);
            const hashRemote = checksum.slice(0, checksum.indexOf(' '));
            const hashTarball = hasha_1.default.fromFileSync(targetPath, {
                algorithm: 'sha256',
            });
            if (hashRemote !== hashTarball) {
                reject(new TypeError(`Updater: checksum mismatch. Got ${hashTarball}, expected ${hashRemote}.`));
                return;
            }
            pino_1.log.info(`Updater: checksum matches`);
            resolve(targetPath);
        });
    });
}
function installationType(commands) {
    if (commands.length < 2) {
        throw new TypeError('Updater: cannot determine installation type');
    }
    // npm install
    if (process.env.NODE_ENV === 'development' ||
        (commands[0].match('node$') !== null &&
            commands[1].match('monika$') !== null)) {
        return 'npm';
    }
    // vercel/pkg
    if (commands[1] === '/snapshot/monika/bin/run.js') {
        return 'binary';
    }
    // npx oclif pack
    if (commands[0].match('node$') !== null &&
        commands[1].match('bin/run.js$') !== null) {
        return 'oclif-pack';
    }
    throw new TypeError(`Updater: unknown installation type. Command: ${process.argv}`);
}
function getPlatform(config) {
    const { platform } = config;
    switch (platform) {
        case 'darwin': {
            return 'macos';
        }
        case 'win32': {
            return 'windows';
        }
        default: {
            return 'linux';
        }
    }
}
//# sourceMappingURL=index.js.map