"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const cockatiel_1 = require("cockatiel");
const macaddress_1 = __importDefault(require("macaddress"));
const os_1 = require("os");
const path_1 = __importDefault(require("path"));
const piscina_1 = __importDefault(require("piscina"));
const probe_1 = require("../components/config/probe");
const config_1 = require("../components/config");
const probe_2 = require("../components/config/validation/validator/probe");
const incident_1 = require("../components/incident");
const alert_message_1 = require("../components/notification/alert-message");
const context_1 = require("../context");
const events_1 = __importDefault(require("../events"));
const flag_1 = require("../flag");
const events_2 = require("../utils/events");
const http_1 = require("../utils/http");
const ip_1 = __importDefault(require("../utils/ip"));
const pino_1 = require("../utils/pino");
const probe_state_1 = require("../utils/probe-state");
const public_ip_1 = require("../utils/public-ip");
const getHandshakeData = async () => {
    if (!(0, public_ip_1.getPublicNetworkInfo)()) {
        await (0, cockatiel_1.retry)(cockatiel_1.handleAll, {
            backoff: new cockatiel_1.ExponentialBackoff(),
        }).execute(async () => {
            try {
                const { city, hostname, isp, privateIp, publicIp } = await (0, public_ip_1.fetchAndCacheNetworkInfo)();
                pino_1.log.info(`[Symon] Monika is running from: ${city} - ${isp} (${publicIp}) - ${hostname} (${privateIp})`);
            }
            catch (error) {
                pino_1.log.error(`[Symon] ${error}. Retrying...`);
                throw error;
            }
        });
    }
    await (0, public_ip_1.getPublicIp)();
    const os = await (0, alert_message_1.getOSName)();
    const macAddress = await macaddress_1.default.one();
    const host = (0, os_1.hostname)();
    const publicIp = public_ip_1.publicIpAddress;
    const privateIp = (0, ip_1.default)();
    const { city, country, isp } = (0, public_ip_1.getPublicNetworkInfo)();
    const { pid } = process;
    const { userAgent: version } = (0, context_1.getContext)();
    return {
        city,
        country,
        hostname: host,
        isp,
        macAddress,
        os,
        pid,
        privateIp,
        publicIp,
        version,
    };
};
class SymonClient {
    constructor({ 'symon-api-version': apiVersion, symonKey = '', symonLocationId = '', symonMonikaId = '', symonReportInterval, symonReportLimit, symonUrl = '', }) {
        this.eventEmitter = (0, events_2.getEventEmitter)();
        this.probeAssignmentTotal = {
            total: 0,
        };
        this.apiKey = symonKey;
        this.url = symonUrl;
        this.httpClient = axios_1.default.create({
            baseURL: `${this.url}/api/${apiVersion}/monika`,
            headers: {
                'x-api-key': this.apiKey,
            },
            timeout: http_1.DEFAULT_TIMEOUT,
        });
        this.locationId = symonLocationId;
        this.monikaId = symonMonikaId;
        this.isMultiNode = apiVersion === flag_1.SYMON_API_VERSION.v2;
        this.reportProbesInterval = symonReportInterval;
        this.reportProbesLimit = symonReportLimit;
        this.worker = new piscina_1.default.Piscina({
            concurrentTasksPerWorker: 1,
            // eslint-disable-next-line unicorn/prefer-module
            filename: path_1.default.join(__dirname, '../../lib/workers/report-to-symon.js'),
            idleTimeout: this.reportProbesInterval,
            maxQueue: 1,
        });
    }
    async initiate() {
        this.monikaId = await this.handshake();
        pino_1.log.info('[Symon] Handshake');
        const probeChangesCheckedAt = new Date();
        await this.fetchProbesAndUpdateConfig();
        this.setProbeChangesCheckedAt(probeChangesCheckedAt);
        this.probeChangesInterval = setInterval(this.fetchAndApplyProbeChanges.bind(this), (0, context_1.getContext)().flags.symonGetProbesIntervalMs);
        if (this.isMultiNode) {
            this.probeAssignmentChangesInterval = setInterval(this.fetchAndApplyProbeAssignmentChanges.bind(this), (0, context_1.getContext)().flags.symonGetProbesIntervalMs);
        }
        this.report().catch((error) => {
            pino_1.log.error(`[Symon] Report failed. ${error.message}`);
        });
        this.eventEmitter.on(events_1.default.probe.notification.willSend, this.willSendEventListener.bind(this));
    }
    async sendStatus({ isOnline }) {
        await this.httpClient({
            data: {
                monikaId: this.monikaId,
                status: isOnline,
            },
            method: 'POST',
            url: '/status',
        });
    }
    async stop() {
        if (this.eventEmitter) {
            this.eventEmitter.removeAllListeners();
        }
        clearInterval(this.probeChangesInterval);
        clearInterval(this.probeAssignmentChangesInterval);
        clearTimeout(this.reportTimeout);
        await this.worker.destroy();
    }
    setProbeChangesCheckedAt(probeChangesCheckedAt) {
        this.probeChangesCheckedAt = probeChangesCheckedAt;
    }
    async fetchAndApplyProbeChanges() {
        const probeChangesCheckedAt = new Date();
        try {
            const probeChanges = await this.probeChanges();
            this.setProbeChangesCheckedAt(probeChangesCheckedAt);
            const hasProbeChanges = probeChanges.length > 0;
            if (!hasProbeChanges) {
                pino_1.log.info(`[Symon] No probe changes since ${this.probeChangesCheckedAt}`);
                return;
            }
            const probeChangesApplyResults = await applyProbeChanges(probeChanges);
            for (const result of probeChangesApplyResults) {
                if (result.status === 'rejected') {
                    pino_1.log.error(`[Symon] Get probe changes since ${this.probeChangesCheckedAt}. ${result.reason}`);
                }
            }
            pino_1.log.info(`[Symon] Get probe changes (${probeChanges.length}) since ${this.probeChangesCheckedAt}`);
        }
        catch (error) {
            pino_1.log.error(`[Symon] Get probe changes since ${this.probeChangesCheckedAt} failed. ${error.message}`);
        }
    }
    willSendEventListener({ probeState, validation, alertId, }) {
        const { data, headers, responseTime, status, error } = validation.response;
        this.httpClient
            .post('/events', {
            monikaId: this.monikaId,
            alertId,
            event: probeState === 'DOWN' ? 'incident' : 'recovery',
            response: {
                body: data,
                headers: typeof headers === 'object' ? headers : {},
                size: typeof headers === 'object' ? headers['content-length'] : undefined,
                status,
                time: responseTime,
                error,
            },
        })
            .then(() => {
            pino_1.log.info(`[Symon] Send ${probeState === 'DOWN' ? 'incident' : 'recovery'} event for Alert ID: ${alertId} succeed`);
        })
            .catch((error) => {
            pino_1.log.error(`[Symon] Send ${probeState === 'DOWN' ? 'incident' : 'recovery'} event for Alert ID: ${alertId} failed.  ${error.message}`);
        });
    }
    async report() {
        // Create a task data object
        const taskData = {
            apiKey: this.apiKey,
            httpClient: this.httpClient,
            monikaId: this.monikaId,
            probeIds: (0, probe_1.getProbes)().map(({ id }) => id),
            reportProbesLimit: this.reportProbesLimit,
            url: this.url,
        };
        try {
            // Submit the task to Piscina
            await this.worker.run(JSON.stringify(taskData));
            pino_1.log.info('[Symon] Report succeed');
        }
        finally {
            this.reportTimeout = setTimeout(() => {
                this.report
                    .bind(this)()
                    .catch((error) => {
                    pino_1.log.error(`[Symon] Report failed. ${error.message}`);
                });
            }, this.reportProbesInterval);
        }
    }
    async probeChanges() {
        const response = await this.httpClient.get(`/${this.monikaId}/probe-changes`, {
            params: { since: this.probeChangesCheckedAt },
        });
        return response.data.data;
    }
    async fetchProbes() {
        const TIMEOUT = 30000;
        return this.httpClient
            .get(`/${this.monikaId}/probes`, {
            timeout: TIMEOUT,
            headers: {
                ...((0, context_1.getContext)().config?.version
                    ? { 'If-None-Match': (0, context_1.getContext)().config?.version }
                    : {}),
            },
            validateStatus(status) {
                return [200, 304].includes(status);
            },
        })
            .then(async (res) => {
            if (!res.data.data) {
                return {
                    probes: (0, probe_1.getProbes)(),
                    hash: res.headers.etag,
                };
            }
            return { probes: res.data.data, hash: res.headers.etag };
        })
            .catch((error) => {
            if (error.isAxiosError) {
                if (error.response) {
                    throw new Error(error.response.data.message);
                }
                if (error.request) {
                    throw new Error('Failed to get probes from Symon');
                }
            }
            throw error;
        });
    }
    async fetchProbesAndUpdateConfig() {
        // Fetch the probes
        const { hash, probes } = await this.fetchProbes();
        await (0, config_1.updateConfig)({ probes, version: hash });
        pino_1.log.info('[Symon] Get probes succeed');
    }
    async handshake() {
        let handshakeData = await getHandshakeData();
        // Check if location id existed and is valid
        if (this.locationId && this.locationId.trim().length > 0) {
            const prevHandshakeData = handshakeData;
            handshakeData = {
                ...prevHandshakeData,
                locationId: this.locationId,
            };
        }
        // Check if Monika id existed and is valid
        if (this.monikaId && this.monikaId.trim().length > 0) {
            const prevHandshakeData = handshakeData;
            handshakeData = {
                ...prevHandshakeData,
                monikaId: this.monikaId,
            };
        }
        return this.httpClient
            .post('/client-handshake', handshakeData)
            .then((res) => res.data?.data.monikaId);
    }
    async fetchAndApplyProbeAssignmentChanges() {
        const probeAssignmentTotal = await this.fetchProbeAssignmentTotal();
        const { total, updatedAt } = probeAssignmentTotal;
        if (total !== this.probeAssignmentTotal.total ||
            updatedAt !== this.probeAssignmentTotal.updatedAt) {
            await this.fetchProbesAndUpdateConfig();
            this.setProbeAssignmentTotal(probeAssignmentTotal);
            pino_1.log.info('[Symon] The probe assignment has been updated');
        }
    }
    setProbeAssignmentTotal(probeAssignmentTotal) {
        this.probeAssignmentTotal = probeAssignmentTotal;
    }
    async fetchProbeAssignmentTotal() {
        const response = await this.httpClient.get(`/${this.monikaId}/probe-assignments/total`);
        return response.data.data;
    }
}
exports.default = SymonClient;
async function applyProbeChanges(probeChanges) {
    return Promise.allSettled(probeChanges.map(async ({ lastEvent, probe, probe_id: probeId, type }) => {
        switch (type) {
            case 'delete':
            case 'disabled': {
                (0, probe_1.deleteProbe)(probeId);
                (0, probe_state_1.removeProbeState)(probeId);
                (0, incident_1.removeIncident)({ probeID: probeId });
                return;
            }
            case 'update': {
                const probes = await (0, probe_2.validateProbes)([
                    lastEvent ? { ...probe, lastEvent } : probe,
                ]);
                (0, probe_1.updateProbe)(probeId, probes[0]);
                (0, probe_state_1.syncProbeStateFrom)(probes[0], 1);
                return;
            }
            case 'add':
            case 'enabled': {
                const probes = await (0, probe_2.validateProbes)([
                    lastEvent ? { ...probe, lastEvent } : probe,
                ]);
                (0, probe_1.addProbe)(probes[0]);
                (0, probe_state_1.syncProbeStateFrom)(probes[0]);
                return;
            }
            default: {
                throw new Error(`Unknown probe changes type (${type}).`);
            }
        }
    }));
}
//# sourceMappingURL=index.js.map