"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendHttpRequestFetch = exports.sendHttpRequest = exports.DEFAULT_TIMEOUT = void 0;
const axios_1 = __importDefault(require("axios"));
const node_http_1 = __importDefault(require("node:http"));
const node_https_1 = __importDefault(require("node:https"));
const undici_1 = require("undici");
// Keep the agents alive to reduce the overhead of DNS queries and creating TCP connection.
// More information here: https://rakshanshetty.in/nodejs-http-keep-alive/
const httpAgent = new node_http_1.default.Agent({ keepAlive: true });
const httpsAgent = new node_https_1.default.Agent({ keepAlive: true });
const insecureHttpsAgent = new node_https_1.default.Agent({
    keepAlive: true,
    rejectUnauthorized: false,
});
exports.DEFAULT_TIMEOUT = 10000;
// Create an instance of axios here so it will be reused instead of creating a new one all the time.
const axiosInstance = axios_1.default.create();
async function sendHttpRequest(config) {
    const { allowUnauthorizedSsl, body, headers, timeout, ...options } = config;
    return axiosInstance.request({
        ...options,
        data: body,
        headers: convertHeadersToAxios(headers),
        timeout: timeout ?? exports.DEFAULT_TIMEOUT,
        httpAgent,
        httpsAgent: allowUnauthorizedSsl ? insecureHttpsAgent : httpsAgent,
    });
}
exports.sendHttpRequest = sendHttpRequest;
function convertHeadersToAxios(headersInit) {
    const headers = {};
    if (headersInit instanceof Headers) {
        // If headersInit is a Headers object
        for (const [key, value] of headersInit.entries()) {
            headers[key] = value;
        }
        return headers;
    }
    if (typeof headersInit === 'object') {
        // If headersInit is a plain object
        for (const [key, value] of Object.entries(headersInit)) {
            headers[key] = value;
        }
        return headers;
    }
    return headers;
}
async function sendHttpRequestFetch(config) {
    const { maxRedirects, timeout, url } = config;
    const controller = new AbortController();
    const { signal } = controller;
    const timeoutId = setTimeout(() => {
        controller.abort();
    }, timeout || exports.DEFAULT_TIMEOUT);
    const fetcher = compileFetch(config, signal);
    return fetchRedirect(url, maxRedirects, fetcher).then((response) => {
        clearTimeout(timeoutId);
        return response;
    });
}
exports.sendHttpRequestFetch = sendHttpRequestFetch;
function compileFetch(config, signal) {
    const { allowUnauthorizedSsl, body, headers, method } = config;
    return (url) => fetch(url, {
        body: body === '' ? undefined : body,
        redirect: 'manual',
        dispatcher: new undici_1.Agent({
            connect: {
                rejectUnauthorized: !allowUnauthorizedSsl,
            },
        }),
        headers,
        keepalive: true,
        method,
        signal,
    });
}
// fetchRedirect handles HTTP status code 3xx returned by fetcher
async function fetchRedirect(url, maxRedirects, fetcher) {
    let redirected = 0;
    let currentResponse;
    let nextUrl = url;
    // do HTTP fetch request at least once
    // then follow redirect based maxRedirects value and HTTP status code 3xx
    do {
        // eslint-disable-next-line no-await-in-loop
        currentResponse = await fetcher(nextUrl);
        // check for HTTP status code 3xx
        const shouldRedirect = currentResponse.status >= 300 && currentResponse.status < 400;
        if (!shouldRedirect)
            break;
        // location header could either be full url, relative path, or absolute path
        // e.g. "https://something.tld", "new/path", "/new/path", respectively
        // refer to : RFC-7231 https://datatracker.ietf.org/doc/html/rfc7231#section-7.1.2
        const newLocation = currentResponse.headers.get('location') || '';
        // try-catch to evaluate if redirect location is a url
        try {
            // when it is valid url, immediately set nextUrl from location header
            nextUrl = new URL(currentResponse.headers.get('location') || '').toString();
        }
        catch {
            // new redirect location is relative / absolute url
            const newEndpoint = newLocation.startsWith('/')
                ? newLocation
                : `/${newLocation}`;
            // parse nextUrl to Node.js URL to get protocol and host
            const parsedURL = new URL(nextUrl);
            // compose nextUrl from parsed protocol, host, and newEndpoint
            nextUrl = `${parsedURL.protocol}//${parsedURL.host}${newEndpoint}`;
        }
        // increment redirected value for while loop
        redirected++;
    } while (redirected <= (maxRedirects || 0));
    return currentResponse;
}
//# sourceMappingURL=http.js.map