"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProbeContext = exports.getProbeState = exports.setProbeFinish = exports.setProbeRunning = exports.removeProbeState = exports.syncProbeStateFrom = exports.initializeProbeStates = void 0;
const xstate_1 = require("xstate");
const events_1 = __importDefault(require("../events"));
const events_2 = require("./events");
const probeStateMachine = (0, xstate_1.createMachine)({
    id: 'probe-state',
    predictableActionArguments: true,
    initial: 'idle',
    states: {
        idle: {
            on: {
                RUN: {
                    target: 'running',
                    actions: 'incrementCycle',
                },
            },
        },
        running: {
            on: {
                FINISH: {
                    target: 'idle',
                    actions: 'setLastFinish',
                },
            },
        },
    },
}, {
    actions: {
        incrementCycle: (0, xstate_1.assign)({
            cycle: (context) => context.cycle + 1,
            lastStart: () => new Date(),
        }),
        setLastFinish: (0, xstate_1.assign)({
            lastFinish: () => new Date(),
        }),
    },
});
const probeInterpreters = new Map();
function initializeProbeStates(probes) {
    probeInterpreters.clear();
    for (const probe of probes) {
        syncProbeStateFrom(probe);
    }
}
exports.initializeProbeStates = initializeProbeStates;
function syncProbeStateFrom(probe, cycle = 0) {
    const getLastStart = (probe) => probe.lastEvent?.createdAt
        ? new Date(probe.lastEvent.createdAt)
        : new Date();
    const getLastFinish = (probe) => probe.lastEvent?.recoveredAt
        ? new Date(probe.lastEvent.recoveredAt)
        : getLastStart(probe);
    const interpreter = (0, xstate_1.interpret)(probeStateMachine.withContext({
        cycle,
        lastStart: getLastStart(probe),
        lastFinish: getLastFinish(probe),
    })).start();
    probeInterpreters.set(probe.id, interpreter);
}
exports.syncProbeStateFrom = syncProbeStateFrom;
function removeProbeState(probeId) {
    probeInterpreters.delete(probeId);
    (0, events_2.getEventEmitter)().emit(events_1.default.probe.finished, probeId);
}
exports.removeProbeState = removeProbeState;
function setProbeRunning(probeId) {
    const interpreter = probeInterpreters.get(probeId);
    if (!interpreter) {
        return '';
    }
    interpreter.send('RUN');
    (0, events_2.getEventEmitter)().emit(events_1.default.probe.ran, probeId);
    return probeId;
}
exports.setProbeRunning = setProbeRunning;
function setProbeFinish(probeId) {
    const interpreter = probeInterpreters.get(probeId);
    interpreter?.send('FINISH');
    (0, events_2.getEventEmitter)().emit(events_1.default.probe.finished, probeId);
}
exports.setProbeFinish = setProbeFinish;
function getProbeState(probeId) {
    const interpreter = probeInterpreters.get(probeId);
    return interpreter?.state?.value;
}
exports.getProbeState = getProbeState;
function getProbeContext(probeId) {
    const interpreter = probeInterpreters.get(probeId);
    return interpreter?.state?.context;
}
exports.getProbeContext = getProbeContext;
//# sourceMappingURL=probe-state.js.map