"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPublicIp = exports.fetchAndCacheNetworkInfo = exports.getPublicNetworkInfo = exports.isConnectedToSTUNServer = exports.publicIpAddress = void 0;
const pino_1 = require("./pino");
const stun_1 = __importDefault(require("stun"));
const os_1 = require("os");
const ip_1 = __importDefault(require("./ip"));
const ping_1 = require("./ping");
const http_1 = require("./http");
const context_1 = require("../context");
const config_1 = require("../components/config");
exports.publicIpAddress = '';
exports.isConnectedToSTUNServer = true;
let publicNetworkInfo;
/**
 * pokeStun sends a poke/request to stun server
 * @returns Promise<string>
 */
async function pokeStun() {
    // for testing, bypass ping/stun server... apparently ping cannot run in github actions
    // reference: https://github.com/actions/virtual-environments/issues/1519
    if ((0, context_1.getContext)().isTest) {
        return '192.168.1.1'; // adding for specific asserts in other tests
    }
    const connection = await (0, ping_1.sendPing)('stun.l.google.com');
    if (connection.alive) {
        const response = await stun_1.default.request('stun.l.google.com:19302');
        return response?.getXorAddress()?.address;
    }
    throw new Error('stun inaccessible'); // could not connect to STUN server
}
async function fetchPublicNetworkInfo() {
    const publicIp = await pokeStun();
    const response = await (0, http_1.sendHttpRequest)({
        url: `http://ip-api.com/json/${publicIp}`,
    });
    const { country, city, isp } = response.data;
    return {
        country,
        city,
        hostname: (0, os_1.hostname)(),
        isp,
        privateIp: (0, ip_1.default)(),
        publicIp,
    };
}
async function setPublicNetworkInfo(publicNetwork) {
    publicNetworkInfo = publicNetwork;
}
function getPublicNetworkInfo() {
    return publicNetworkInfo;
}
exports.getPublicNetworkInfo = getPublicNetworkInfo;
// cache location & ISP info
async function fetchAndCacheNetworkInfo() {
    const publicNetwork = await fetchPublicNetworkInfo();
    await setPublicNetworkInfo(publicNetwork);
    return publicNetwork;
}
exports.fetchAndCacheNetworkInfo = fetchAndCacheNetworkInfo;
/**
 * getPublicIP sends a request to stun server getting IP address
 * @returns Promise<any>
 */
async function getPublicIp() {
    const { flags } = (0, context_1.getContext)();
    const time = new Date().toISOString();
    const isSymonMode = (0, config_1.isSymonModeFrom)(flags);
    try {
        const address = await pokeStun();
        if (address) {
            exports.isConnectedToSTUNServer = true;
            if (flags.verbose || isSymonMode) {
                // reveal address info?
                exports.publicIpAddress = address;
                pino_1.log.info(`${time} - Connected to STUN Server. Monika is running from: ${address}`);
            }
            else {
                pino_1.log.info(`${time} - Connected to STUN Server. Monika is running.`);
            }
        }
    }
    catch {
        exports.isConnectedToSTUNServer = false;
        pino_1.log.warn(`${time} STUN Server is temporarily unreachable. Check network.`);
    }
}
exports.getPublicIp = getPublicIp;
//# sourceMappingURL=public-ip.js.map