"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
exports.send = exports.validator = void 0;
const tslib_1 = require("tslib");
const axios_1 = tslib_1.__importStar(require("axios"));
const joi_1 = tslib_1.__importDefault(require("joi"));
const database_1 = require("./database");
const http_1 = tslib_1.__importDefault(require("http"));
const https_1 = tslib_1.__importDefault(require("https"));
const BASE_URL = 'https://api.instatus.com';
exports.validator = joi_1.default.object({
    apiKey: joi_1.default.string().required().label('API Key'),
    pageID: joi_1.default.string().required().label('Page ID'),
}).required();
async function send(notificationData, message) {
    const { meta, summary } = message;
    const { probeID, url, type } = meta;
    switch (type) {
        case 'incident': {
            await createIncident(notificationData, {
                probeID,
                url,
            }, summary);
            break;
        }
        case 'recovery': {
            await updateIncident(notificationData, {
                probeID,
                url,
            });
            break;
        }
        default: {
            break;
        }
    }
}
exports.send = send;
async function createIncident(instatusConfig, { probeID, url }, message = "We're currently experiencing an issue with the website") {
    var _a, _b, _c;
    const status = 'INVESTIGATING';
    const incident = await (0, database_1.findIncident)({
        probeID,
        status,
        url,
    });
    // prevent duplicate incident
    if (incident) {
        return;
    }
    try {
        const components = await getComponents(instatusConfig);
        const componentIDs = components.map(({ id }) => id);
        const componentID = componentIDs[0];
        const started = new Date();
        const statuses = componentIDs.map((id) => ({
            id,
            status: 'MAJOROUTAGE',
        }));
        const data = {
            name: 'Service is down',
            message,
            components: [componentID],
            started,
            status,
            notify: true,
            statuses,
        };
        const { apiKey, pageID } = instatusConfig;
        const incidentID = await axios_1.default
            .post(`${BASE_URL}/v1/${pageID}/incidents/`, data, getAxiosConfig(apiKey))
            .then((res) => { var _a; return (_a = res === null || res === void 0 ? void 0 : res.data) === null || _a === void 0 ? void 0 : _a.id; });
        await (0, database_1.insertIncident)({ incidentID, probeID, status, url });
    }
    catch (error) {
        const axiosError = error instanceof axios_1.AxiosError ? error : new axios_1.AxiosError();
        throw new Error(`${axiosError === null || axiosError === void 0 ? void 0 : axiosError.message}${((_a = axiosError === null || axiosError === void 0 ? void 0 : axiosError.response) === null || _a === void 0 ? void 0 : _a.data)
            ? `. ${(_c = (_b = axiosError === null || axiosError === void 0 ? void 0 : axiosError.response) === null || _b === void 0 ? void 0 : _b.data) === null || _c === void 0 ? void 0 : _c.message}`
            : ''}`);
    }
}
async function updateIncident(instatusConfig, { probeID, url }) {
    var _a, _b, _c;
    const status = 'RESOLVED';
    const components = await getComponents(instatusConfig);
    const componentIDs = components.map(({ id }) => id);
    const componentID = componentIDs[0];
    const started = new Date();
    const statuses = componentIDs.map((id) => ({
        id,
        status: 'OPERATIONAL',
    }));
    const data = {
        name: 'Service is up',
        components: [componentID],
        started,
        status,
        statuses,
    };
    const incident = await (0, database_1.findIncident)({
        probeID,
        status: 'INVESTIGATING',
        url,
    });
    if (!incident) {
        throw new Error('Instatus notification: Incident is not found.');
    }
    const { incident_id: incidentID } = incident;
    try {
        const { apiKey, pageID } = instatusConfig;
        await axios_1.default.patch(`${BASE_URL}/v1/${pageID}/incidents/${incidentID}`, data, getAxiosConfig(apiKey));
    }
    catch (error) {
        const axiosError = error instanceof axios_1.AxiosError ? error : new axios_1.AxiosError();
        throw new Error(`${axiosError.message}${((_a = axiosError === null || axiosError === void 0 ? void 0 : axiosError.response) === null || _a === void 0 ? void 0 : _a.data)
            ? `. ${(_c = (_b = axiosError === null || axiosError === void 0 ? void 0 : axiosError.response) === null || _b === void 0 ? void 0 : _b.data) === null || _c === void 0 ? void 0 : _c.message}`
            : ''}`);
    }
    await (0, database_1.updateIncident)({ incidentID, status });
}
async function getComponents({ apiKey, pageID, }) {
    const componentsResponse = await axios_1.default.get(`${BASE_URL}/v1/${pageID}/components`, getAxiosConfig(apiKey));
    return componentsResponse.data;
}
function getAxiosConfig(apiKey) {
    return {
        // 10 sec timeout
        timeout: 10000,
        // keepAlive pools and reuses TCP connections, so it's faster
        httpAgent: new http_1.default.Agent({ keepAlive: true }),
        httpsAgent: new https_1.default.Agent({ keepAlive: true }),
        // follow up to 10 HTTP 3xx redirects
        maxRedirects: 10,
        headers: {
            Authorization: `Bearer ${apiKey}`,
            'Content-Type': 'application/json',
        },
    };
}
